# from context import BASEDIR
from source.excel import Excel
import argparse,json

def main():
    parser=argparse.ArgumentParser(description="For munipulate excel and get data source...")
    
    parser.add_argument("-e", "--excel", help="Input excel file name")
    parser.add_argument("-a", "--add", help="add up two excels, and output to another excel",nargs="+")
    parser.add_argument("-s", "--sub", help="sub two excels, and output to another excel",nargs="+")
    parser.add_argument("-t", "--to", help="Output excel file name")
    parser.add_argument("-po", "--protection_off", help="Protection off. default is On",action='store_true')
    parser.add_argument("-j", "--json", help="need json output",action='store_true')
    parser.add_argument("-d", "--dict", help="need dict output",action='store_true')
    
    
    args = parser.parse_args()
    protection=False if args.protection_off else True

    if args.add and args.to:
        if len(args.add)!=2:
            raise ValueError("Must two excel files after -a ")
        e1=Excel(args.add[0])
        e2=Excel(args.add[1])
        try:
            e=e1+e2
            e.makeFormatedExcel(args.to,protection=protection)
            if args.json:
                print(e.json)
            if args.dict:
                print(e.dict)  
        except ValueError as e:
            print(str(e))
            
    if args.sub and args.to:
        if len(args.sub)!=2:
            raise ValueError("Must two excel files after -a ")
        e1=Excel(args.sub[0])
        e2=Excel(args.sub[1])
        try:
            e=e1-e2
            e.makeFormatedExcel(args.to,protection=protection)
            if args.json:
                print(e.json)
            if args.dict:
                print(e.dict) 
            
        except ValueError as e:
            print(str(e)) 
    
    if args.excel and args.json:
        e=Excel(args.excel)
        print(json.dumps(e.json,indent=3,default=str))

if __name__=="__main__":
    main()






from pydantic import BaseModel, EmailError, EmailStr,validator
from model.common.utils import Duration
from datetime import date
from typing import Optional,List
from model.common.mixins import DurationMixin

class Advertisement(BaseModel,DurationMixin):
    method:str
    media:str
    website:str
    advertisement_id:Optional[str]
    start_date:date	
    end_date:date
    
    @property
    def days(self):
        return Duration(self.start_date,date.today()).days
    
    def daysOnDate(self,end_date=date.today()):
        return Duration(self.start_date,end_date).daysOnDate(end_date)
    
    @validator("end_date")
    def endDateBigger(cls,end_date,values):
        start_date=values.get('start_date')
        the_date=end_date if end_date else date.today()
        if (the_date-start_date).days<=0:
            raise ValueError(f'End date {the_date} is earlier than start date {start_date}')
        end_date=end_date if end_date else "Present"
        return end_date
    

class Advertisements():
    def __init__(self,advertisments:List[Advertisement]):
        self.advertisements=advertisments
        
    @property
    def amount(self):
        return len(self.advertisements)
    
    @property
    def earliest(self):
        return min([ day.start_date for day in self.advertisements])
    
    @property
    def latest(self):
        return max([ day.start_date for day in self.advertisements])
    
    
    @property
    def earliest_say(self):
        return self.earliest.strftime("%b %d, %Y")
    
    @property
    def min_days(self):
        return (date.today()-self.latest).days
    
    @property
    def max_days(self):
        return (date.today()-self.earliest).days
    
    @property
    def summary(self):
        medias=[ adv.media for adv in self.advertisements]
        return (f"""We posted {self.amount} job advertisements on {', '.join(medias[:-1])+", and "+medias[-1]} since {self.earliest_say} for at least {self.min_days} days.""")
        
class InterviewRecord(BaseModel):
    candidate:str
    canadian_status:str
    interviewed:Optional[bool]
    record:str
    offered:Optional[bool]
    accepted:Optional[bool]
    
    @property
    def is_interviewed(self):
        return "Yes" if self.interviewed else "No"
    
    @property
    def is_offered(self):
        return "Yes" if self.offered else "No"
    
    @property
    def is_accepted(self):
        return "Yes" if self.accepted else "No"
    
    @property
    def is_canadian(self):
        return True if self.canadian_status.upper()=="CITIZEN" or  self.canadian_status.upper()=="PR" else False
    
    @property
    def is_unknown(self):
        return True if self.canadian_status.upper()=="UNKNOWN"  else False
    
    @property
    def is_foreigner(self):
        return True if self.canadian_status.upper()=="FOREIGNER"  else False

class InterviewRecords():
    def __init__(self,interview_records:List[InterviewRecord]):
        self.interview_records=interview_records
        
    @property
    def resume_num(self):
        return len(self.interview_records)
    
    @property
    def canadian_num(self):
        return len([record for record in self.interview_records if record.is_canadian])
    
    @property
    def unknown_num(self):
        return len([record for record in self.interview_records if record.is_unknown])
    
    @property
    def foreigner_num(self):
        return len([record for record in self.interview_records if record.is_foreigner])
    
    @property
    def total_canadian(self):
        return self.unknown_num+self.canadian_num
    
    @property
    def total_interviewed_canadians(self):
        return len([ applicant for applicant in self.interview_records if applicant.interviewed==True])
    
    @property
    def canadian_records(self):
        records=[]
        i=1
        for applicant in self.interview_records:
            if applicant.is_canadian or applicant.is_unknown:
                records.append({**{'no':i},**applicant.dict()})
                i+=1
        return records
    
    @property
    def summary(self):
        return (f"""Totally, we have received {self.resume_num} resumes consisting of {self.canadian_num } Canadian(s), {self.unknown_num } Unknown status, and {self.foreigner_num } foreign national(s) (who claimed their nationality, work permit, or study permit holder or visitors) during the first screening. We assume that all candidates with unknown status are also Canadians, so there are {self.total_canadian } Canadian(s), among which {self.total_interviewed_canadians} were reached out for interview.""")
    
class RecruitmentSummary(BaseModel):
    apply_email:EmailStr
    referral:Optional[str]
    reply2apply:bool
    emails_for_making_interview:bool
    interview_date:date
    interview_way:str
    interviewer_name:str
    interviewer_position:str
    interview_record:bool
    interview_process_evidence:bool
    emails_for_certificates:bool
    emais_for_references:bool
    reference_checked:bool
    reference_check_evidence:bool
    joboffer_email:bool
    joboffer_email_reply:bool
    after_offer_coomunication:bool
    reasons_not_hire_canadians:str




from typing import List, Union
from webform.models.definition import Action
from webform.models.bcpnpmodel_app import BcpnpModelApp, BcpnpEEModelApp
from webform.bcpnp.dashboard import DashboardApp
from model.common.employmenthistory import EmploymentHistory
from datetime import datetime
from webform.bcpnp.data import country_map
from model.common.family import FamilyMembers
from webform.bcpnp.data import marriage_map, province_map, family_imm_status_canada_map


class Family:
    def __init__(self, person: Union[BcpnpModelApp, BcpnpEEModelApp]):
        self.person = person
        self.spouse = FamilyMembers(self.person.family).spouse
        self.dependants = FamilyMembers(self.person.family).dependants
        self.mother = FamilyMembers(self.person.family).mother
        self.father = FamilyMembers(self.person.family).father
        self.siblings = FamilyMembers(self.person.family).siblings

    @property
    def spouse_details(self):
        is_married = [
            {
                "action_type": Action.Radio.value,
                "label": "Are you currently married or in a common-law relationship? ",
                "id": "#syncA_App_MaritalStatus-Yes"
                if self.person.marriage.marital_status in ["Common-Law", "Married"]
                else "#syncA_App_MaritalStatus-No",
            }
        ]
        sp = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "Family name",
                    "id": "#syncA_App_Spouse_Lname",
                    "value": self.person.marriage.sp_last_name,
                    "required": True,
                    "length": 100,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Given name",
                    "id": "#syncA_App_Spouse_Fname",
                    "value": self.person.marriage.sp_first_name,
                    "required": True,
                    "length": 100,
                },
                {
                    "action_type": Action.Select.value,
                    "label": "Gender",
                    "id": "#syncA_App_Spouse_Sex",
                    "value": "Male"
                    if self.person.personal.sex == "Female"
                    else "Female",
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Date of birth",
                    "id": "#syncA_App_Spouse_DOB",
                    "value": self.spouse.date_of_birth,
                    "required": True,
                    "length": 100,
                },
                {
                    "action_type": Action.Select.value,
                    "label": "Country of birth",
                    "id": "#syncA_App_Spouse_BirthPlace",
                    "value": country_map[self.spouse.birth_country],
                },
                {
                    "action_type": Action.Select.value,
                    "label": "Country of citizen",
                    "id": "#syncA_App_Spouse_Citizenship",
                    "value": country_map[self.spouse.country_of_citizenship],
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Present address",
                    "id": "#BCPNP_App_Spouse_Addr",
                    "value": self.spouse.address,
                    "required": True,
                    "length": 200,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Date into marriage or common-law",
                    "id": "#BCPNP_App_Spouse_MarriageDate",
                    "value": self.person.marriage.married_date,
                    "required": True,
                    "length": 10,
                },
            ]
            if self.person.marriage.marital_status in ["Common-Law", "Married"]
            else []
        )
        in_canada = [
            {
                "action_type": Action.Radio.value,
                "label": "Is your spouse or common-law partner in Canada",
                "id": "#BCPNP_App_spouse_InCanada-Yes"
                if self.person.marriage.sp_in_canada
                else "#BCPNP_App_spouse_InCanada-No",
            }
        ]
        in_canada_status = (
            [
                {
                    "action_type": Action.Select.value,
                    "label": "Status in Canada",
                    "id": "#BCPNP_App_Spouse_InCanada_Status",
                    "value": self.person.marriage.sp_canada_status,
                },
            ]
            if self.person.marriage.sp_in_canada
            else []
        )

        other = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "If other, specify",
                    "id": "#BCPNP_App_Spouse_InCanada_Status_Other",
                    "value": self.person.marriage.sp_in_canada_other,
                    "required": True,
                    "length": 10,
                }
            ]
            if self.person.marriage.sp_canada_status == "Other"
            else []
        )

        status_details = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "Date spouse or common-law partner documents expire",
                    "id": "#BCPNP_syncA_App_Spouse_Status_Expires",
                    "value": self.person.marriage.sp_canada_status_end_date,
                    "required": True,
                    "length": 10,
                }
            ]
            if self.person.marriage.sp_canada_status
            in ["Worker", "Student", "Visitor", "Other"]
            else []
        )

        # spouse work
        in_canada_work = (
            [
                {
                    "action_type": Action.Radio.value,
                    "label": "Is your spouse or common-law partner currently working?",
                    "id": "#BCPNP_App_spousePartnerWorking-Yes"
                    if self.person.marriage.sp_in_canada_work
                    else "#BCPNP_App_spousePartnerWorking-No",
                },
            ]
            if self.person.marriage.sp_in_canada
            else []
        )

        work_details = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "Spouse or common-law partner occupation",
                    "id": "#BCPNP_App_Spouse_Occupation",
                    "value": self.person.marriage.sp_canada_occupation,
                    "required": True,
                    "length": 200,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Spouse or common-law partner employer",
                    "id": "#BCPNP_App_Spouse_Employer",
                    "value": self.person.marriage.sp_canada_employer,
                    "required": True,
                    "length": 200,
                },
            ]
            if self.person.marriage.sp_in_canada
            and self.person.marriage.sp_in_canada_work
            else []
        )

        return (
            is_married
            + sp
            + in_canada
            + in_canada_status
            + other
            + status_details
            + in_canada_work
            + work_details
        )

    def dependant_block(self, dependant, index):
        return [
            {
                "action_type": Action.Input.value,
                "label": "Family name",
                "id": f"#BCPNP_App_Child_Lname-{index}",
                "value": dependant.last_name,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Given name",
                "id": f"#BCPNP_App_Child_Fname-{index}",
                "value": dependant.first_name,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Date of birth",
                "id": f"#BCPNP_App_Child_DOB-{index}",
                "value": dependant.date_of_birth,
                "required": True,
                "length": 10,
            },
            {
                "action_type": Action.Select.value,
                "label": "Country of birth",
                "id": f"#BCPNP_App_Child_BirthPlace-{index}",
                "value": country_map[dependant.birth_country],
            },
            {
                "action_type": Action.Select.value,
                "label": "Citizen country",
                "id": f"#BCPNP_App_Child_Citizenship-{index}",
                "value": country_map[dependant.country_of_citizenship],
            },
            {
                "action_type": Action.Input.value,
                "label": "Present address",
                "id": f"#BCPNP_App_Child_Addr-{index}",
                "value": dependant.address,
                "required": True,
                "length": 300,
            },
        ]

    def sibling_block(self, sibling, index):
        sibling_deceased = True if sibling.date_of_death else False
        basic = [
            {
                "action_type": Action.Input.value,
                "label": "Family name",
                "id": f"#BCPNP_App_Sibling_Lname-{index}",
                "value": sibling.last_name,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Given name",
                "id": f"#BCPNP_App_Sibling_Fname-{index}",
                "value": sibling.first_name,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Date of birth",
                "id": f"#BCPNP_App_Sibling_DOB-{index}",
                "value": sibling.date_of_birth,
                "required": True,
                "length": 10,
            },
            {
                "action_type": Action.Select.value,
                "label": "Country of birth",
                "id": f"#BCPNP_App_Sibling_BirthPlace-{index}",
                "value": country_map[sibling.birth_country],
            },
            {
                "action_type": Action.Select.value,
                "label": "Marital stutus",
                "id": f"#BCPNP_App_Sibling_MaritalStatus-{index}",
                "value": marriage_map[sibling.marital_status],
            },
            {
                "action_type": Action.Checkbox.value,
                "label": "Deceased?",
                "id":f"#BCPNP_App_Sibling_Deceased-{index}" ,
                "value": sibling_deceased,
            },
        ]

        address = (
            []
            if sibling_deceased
            else [
                {
                    "action_type": Action.Input.value,
                    "label": "Present address",
                    "id": f"#BCPNP_App_Sibling_Addr-{index}",
                    "value": sibling.address,
                    "required": True,
                    "length": 300,
                }
            ]
        )
        return basic + address

    def other_block(self, other, index):
        return [
            {
                "action_type": Action.Input.value,
                "label": "Family name",
                "id": f"#BCPNP_App_CANFam_Lname-{index}",
                "value": other.last_name,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Given name",
                "id": f"#BCPNP_App_CANFam_Fname-{index}",
                "value": other.first_name,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Select.value,
                "label": "Gender",
                "id": f"#BCPNP_App_CANFam_Sex-{index}",
                "value": other.sex,
            },
            {
                "action_type": Action.Input.value,
                "label": "Relationship to applicant",
                "id": f"#BCPNP_App_CANFam_RelType-{index}",
                "value": other.relationship,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "City/town of relative",
                "id": f"#BCPNP_App_CANFam_City-{index}",
                "value": other.city,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Select.value,
                "label": "Province",
                "id": f"#BCPNP_App_CANFam_Province-{index}",
                "value": province_map[other.province],
            },
            {
                "action_type": Action.Select.value,
                "label": "Immigration status in Canada",
                "id": f"#BCPNP_App_CANFam_Status-{index}",
                "value": family_imm_status_canada_map[other.status],
            },
            {
                "action_type": Action.Input.value,
                "label": "Years in Canada",
                "id": f"#BCPNP_App_CANFam_CANYears-{index}",
                "value": str(other.years_in_canada),
                "required": True,
                "length": 10,
            },
        ]

    @property
    def dependant_children(self):
        has_dependants = {
            "action_type": Action.Radio.value,
            "label": "Do you have any children that meet Immigration, Refugees and Citizenship Canada (IRCC) definition of dependent child? ",
            "id": "#BCPNP_App_HaveDepChildren-Yes"
            if len(self.dependants) > 0
            else "#BCPNP_App_HaveDepChildren-No",
        }

        return [
            has_dependants,
            {
                "action_type": Action.RepeatSection.value,
                "button_id": "body > div > main > div.layout-container > div > div > div > uf-form > div > ng-transclude > div > uf-tabset > div > div.tab-content > ng-transclude > uf-tab:nth-child(4) > div > uf-panel:nth-child(3) > div > uf-panel-body > div > uf-panel > div > uf-panel-body > div > div:nth-child(2) > uf-clone-repeatable > a > i",
                "value": self.repeat_blocks(self.dependants, self.dependant_block),
            },
        ]

    @property
    def number_of_members(self):
        spouse = 1 if self.spouse else 0
        number = str(
            len(self.dependants) + spouse + 1
        )  # applicant self, plus spouse, and dependant children
        return [
            {
                "action_type": Action.Input.value,
                "label": "Family members in application",
                "id": "#syncA_App_FamMembers",
                "value": number,
                "required": True,
                "length": 2,
            }
        ]

    @property
    def parents(self):
        mother_deceased = True if self.mother.date_of_death else False
        father_deceased = True if self.father.date_of_death else False
        mother = [
            {
                "action_type": Action.Input.value,
                "label": "Family name",
                "id": "#BCPNP_App_Mother_Lname",
                "value": self.mother.last_name,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Given name",
                "id": "#BCPNP_App_Mother_Fname",
                "value": self.mother.first_name,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Date of birth",
                "id": "#BCPNP_App_Mother_DOB",
                "value": self.mother.date_of_birth,
                "required": True,
                "length": 10,
            },
            {
                "action_type": Action.Select.value,
                "label": "Country of birth",
                "id": "#BCPNP_App_Mother_BirthPlace",
                "value": country_map[self.mother.birth_country],
            },
            {
                "action_type": Action.Checkbox.value,
                "label": "Deceased?",
                "id": "#BCPNP_App_Mother_Deceased",
                "value": mother_deceased,
            },
        ]
        mother_present_address = (
            []
            if mother_deceased
            else [
                {
                    "action_type": Action.Input.value,
                    "label": "Present address",
                    "id": "#BCPNP_App_Mother_Addr",
                    "value": self.mother.address,
                    "required": True,
                    "length": 200,
                }
            ]
        )

        father = [
            {
                "action_type": Action.Input.value,
                "label": "Family name",
                "id": "#BCPNP_App_Father_Lname",
                "value": self.father.last_name,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Given name",
                "id": "#BCPNP_App_Father_Fname",
                "value": self.father.first_name,
                "required": True,
                "length": 100,
            },
            {
                "action_type": Action.Input.value,
                "label": "Date of birth",
                "id": "#BCPNP_App_Father_DOB",
                "value": self.father.date_of_birth,
                "required": True,
                "length": 10,
            },
            {
                "action_type": Action.Select.value,
                "label": "Country of birth",
                "id": "#BCPNP_App_Father_BirthPlace",
                "value": country_map[self.father.birth_country],
            },
            {
                "action_type": Action.Checkbox.value,
                "label": "Deceased?",
                "id": "#BCPNP_App_Father_Deceased",
                "value": father_deceased,
            },
        ]
        father_present_address = (
            []
            if father_deceased
            else [
                {
                    "action_type": Action.Input.value,
                    "label": "Present address",
                    "id": "#BCPNP_App_Father_Addr",
                    "value": self.father.address,
                    "required": True,
                    "length": 200,
                }
            ]
        )

        return mother + mother_present_address + father + father_present_address

    @property
    def siblings_details(self):
        has_siblings = {
            "action_type": Action.Radio.value,
            "label": "Do you have any Siblings, including half- and step- Siblings?  ",
            "id": "#BCPNP_App_HaveSiblings-Yes"
            if len(self.dependants) > 0
            else "#BCPNP_App_HaveSiblings-No",
        }

        return [
            has_siblings,
            {
                "action_type": Action.RepeatSection.value,
                "button_id": "body > div > main > div.layout-container > div > div > div > uf-form > div > ng-transclude > div > uf-tabset > div > div.tab-content > ng-transclude > uf-tab:nth-child(4) > div > uf-panel:nth-child(7) > div > uf-panel-body > div > uf-panel > div > uf-panel-body > div > div:nth-child(2) > uf-clone-repeatable > a > ng-transclude > span",
                "value": self.repeat_blocks(self.siblings, self.sibling_block),
            },
        ]

    @property
    def others(self):
        has_others = {
            "action_type": Action.Radio.value,
            "label": "Do you have any family members in Canada, excluding those included in this application?  ",
            "id": "#BCPNP_App_HaveCANFam-Yes"
            if len(self.person.canadarelative) > 0
            else "#BCPNP_App_HaveCANFam-No",
        }

        return [
            has_others,
            {
                "action_type": Action.RepeatSection.value,
                "button_id": "body > div > main > div.layout-container > div > div > div > uf-form > div > ng-transclude > div > uf-tabset > div > div.tab-content > ng-transclude > uf-tab:nth-child(4) > div > uf-panel:nth-child(8) > div > uf-panel-body > div > uf-panel > div > uf-panel-body > div > div:nth-child(2) > uf-clone-repeatable > a > ng-transclude > span",
                "value": self.repeat_blocks(
                    self.person.canadarelative, self.other_block
                ),
            },
        ]

    def repeat_blocks(self, sources: List, processor):
        blocks = []
        for index, source in enumerate(sources):
            blocks.append(processor(source, index))
        return blocks

    def fill(self):
        # dashboard = DashboardApp()
        actions = (
            # dashboard.jump("Family")
            self.spouse_details
            + self.dependant_children
            # + self.number_of_members
            + self.parents
            + self.siblings_details
            + self.others
            # + dashboard.save
        )
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Family",
                "actions": actions,
                "id": "body > div > main > div.layout-container > div > div > div > uf-form > div > ng-transclude > div > uf-tabset > div > ul > li:nth-child(5) > a",
                "next_page_tag":"#syncA_App_FullTimeEmpOffer-No"
            }
        ]



from model.tr.pdfformmodel import PdfFormModel
from model.tr.d1295 import data_model, convert_model,remove_model

class F1295Model(PdfFormModel):
    def __init__(self,xml_file):
        model={'data_model':data_model,'convert_model':convert_model,'remove_model':remove_model}
        super().__init__(xml_file,model)

    def _getSpecial(self):
            # work 
        occupation_info=[e for e in self.tree.iterfind('Page3/PageWrapper/Occupation')][0]
        occupations=[]
        for i in [1,2,3]:
            occupation_item=occupation_info.find('OccupationRow'+str(i))
            occupation=self._getSubItem(occupation_item,self.data_model['occupation'])
            occupations.append(occupation)

        self.data['occupations']=occupations        



from typing import List
from model.common.commonmodel import CommonModel
from model.bcpnp.data import Bcpnp, Contact, General,JobOffer,ErAddress
from model.common.jobposition import Position
from model.common.rcic import Rcic
from model.common.advertisement import Advertisement,Advertisements,InterviewRecord,InterviewRecords,RecruitmentSummary
from model.common.person import Person,PersonalAssess
from model.common.contact import Contacts
from model.common.address import Addresses
from model.common.phone import Phone,Phones
from datetime import date
from model.common.xmlfiller import XmlFiller
import os


class Personal(Person):
    def __str__(self):
        return self.full_name
        
class JobOfferFormModel(CommonModel):
    eraddress:List[ErAddress]
    phone:List[Phone]
    general:General
    contact:List[Contact]
    position:Position
    personal:Personal
    joboffer:JobOffer
    personalassess:PersonalAssess
    bcpnp:Bcpnp
    rcic:Rcic
    advertisement:List[Advertisement]
    interviewrecord:List[InterviewRecord]
    recruitmentsummary:RecruitmentSummary

    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self,excels=None,output_excel_file=None):
        if output_excel_file:
            excels=self.getExcels(['excel/er.xlsx','excel/pa.xlsx','excel/recruitment.xlsx','excel/bcpnp.xlsx','excel/rep.xlsx'])
        else:
            if excels is None and len(excels)==0:
                raise ValueError('You must input excel file list as source data for validation')
        # call parent class for validating
        super().__init__(excels,output_excel_file,globals())
    
    @property
    def work_location(self):
        addresses=Addresses(self.eraddress)
        return addresses.working
    
    @property
    def phones(self):
        return Phones(self.phone)
    
    @property
    def selected_contact(self):
        contacts=Contacts(self.contact)
        return contacts.preferredContact
    
    @property
    def interviews(self):
        return InterviewRecords(self.interviewrecord)
    
    @property
    def advertisements(self):
        return Advertisements(self.advertisement)
    
    @property
    def businessaddress(self):
        eraddress=Addresses(self.eraddress)
        return eraddress.business
    
    @property
    def mailingaddress(self):
        eraddress=Addresses(self.eraddress)
        return eraddress.mailing
    
    @property
    def person(self):
        return {
            "first_name":self.personal.first_name,
            "last_name":self.personal.last_name,
            "full_name":self.personal.full_name,
            "attributive":self.personal.attributive,
            "object":self.personal.object,
            "subject":self.personal.subject,
            'short_name':self.personal.short_name,
            'why_tfw':self.personalassess.why_qualified_say
        }

class JobOfferFormDocxAdaptor():
    def __init__(self,jobofferform_obj:JobOfferFormModel):
        self.jobofferform_obj=jobofferform_obj
        
    def re_generate_dict(self):
        summary_info={
            'personal':self.jobofferform_obj.person,
            "date_of_offer":self.jobofferform_obj.joboffer.date_of_offer,
            "work_start_date":self.jobofferform_obj.joboffer.start_date_say,
            'joboffer_date':self.jobofferform_obj.joboffer.date_of_offer,
            'annual_rate':float(self.jobofferform_obj.joboffer.annual_rate),
            'hourly_rate':float(self.jobofferform_obj.joboffer.hourly_rate),
            'work_location':self.jobofferform_obj.work_location,
            'b_address':self.jobofferform_obj.businessaddress.line1,
            'b_city':self.jobofferform_obj.businessaddress.city,
            'b_province':self.jobofferform_obj.businessaddress.province,
            'b_country':self.jobofferform_obj.businessaddress.country,
            'b_postcode':self.jobofferform_obj.businessaddress.post_code,
            'm_address':self.jobofferform_obj.mailingaddress.line1,
            'm_city':self.jobofferform_obj.mailingaddress.city,
            'm_province':self.jobofferform_obj.mailingaddress.province,
            'm_country':self.jobofferform_obj.mailingaddress.country,
            'm_postcode':self.jobofferform_obj.mailingaddress.post_code,
            'w_address':self.jobofferform_obj.work_location.line1,
            'w_city':self.jobofferform_obj.work_location.city,
            'w_province':self.jobofferform_obj.work_location.province,
            'w_country':self.jobofferform_obj.work_location.country,
            'w_postcode':self.jobofferform_obj.work_location.post_code,
            'date_signed':date.today(),
            # position 3c
            #TODO: is new position , ...
            # "under_cba":
            "which_union":self.jobofferform_obj.position.which_union,
            "lmia_refused":"Yes_5"  if self.jobofferform_obj.position.lmia_refused else "No_5",
            "lmia_refused_reason": self.jobofferform_obj.position.lmia_refused_reason if self.jobofferform_obj.position.lmia_refused else "",
            # "other_language_required":self.jobofferform_obj.joboffer.other_language_required #TODO: ,
            "what_language":self.jobofferform_obj.joboffer.reason_for_other,
            
            # "license_required":
            "which_license":self.jobofferform_obj.joboffer.license_description,
            
            # 4. recruitment summary
            "active_recruitment":"Yes_7" if self.jobofferform_obj.advertisements.amount>0 else "No_7",
            "min_days":self.jobofferform_obj.advertisements.min_days,
            "total_applicant":self.jobofferform_obj.interviews.resume_num,
            'recruitment_summary':self.jobofferform_obj.advertisements.summary+self.jobofferform_obj.interviews.summary+self.jobofferform_obj.position.how_did_you_find,

            "contact_email":self.jobofferform_obj.selected_contact.email,
            "contact_last_name":self.jobofferform_obj.selected_contact.last_name,
            "contact_first_name":self.jobofferform_obj.selected_contact.first_name,
            "contact_phone":self.jobofferform_obj.selected_contact.phone,
            'contact_position':self.jobofferform_obj.selected_contact.position,
            'business_phone':self.jobofferform_obj.phones.business.international_format_full,
            
        }
        return {**self.jobofferform_obj.dict(),**summary_info}

    def make(self,output_xml):
        path=os.path.abspath(os.path.join(os.path.dirname(__file__), os.path.pardir))
        template_path=path+"/template/xml/bcpnp_job_offer_form.xml"            
        xf=XmlFiller(template_path,self.re_generate_dict())
        xf.save(output_xml)



from pydantic import BaseModel
from typing import Optional,List

class Address(BaseModel):
    variable_type:str    # this is the identifier variable 
    display_type: str # actually it is the prompt information
    po_box:Optional[str]
    unit:Optional[str]
    street_number:Optional[str]
    street_name:Optional[str]
    district:Optional[str]
    city:Optional[str]
    province:Optional[str]
    country:Optional[str]
    post_code:Optional[str]
    
    @property
    def full_address(self):
        fa=self.po_box+' ' if self.po_box else ''
        fa+=self.unit+' ' if self.unit else ''
        fa+=self.street_number+" " if self.street_number else ""
        fa+=self.street_name
        fa+=", "+self.district+', ' if self.district else ""
        fa+=self.city+', '+self.province+', '+self.country+' '+self.post_code
        return fa
    
    @property
    def line1(self):
        l1=self.po_box+' ' if self.po_box else ''
        l1+=self.unit+' ' if self.unit else ''
        l1+=self.street_number+" " if self.street_number else ""
        l1+=self.street_name
        l1+=", "+self.district+', ' if self.district else ""
        return l1

    @property
    def line2(self):
        l2=self.city+', '+self.province+', '+self.country+' '+self.post_code
        return l2
    
    @property
    def street_address(self):
        l1=self.street_number+" " if self.street_number else ""
        l1+=self.street_name if self.street_name else ""
        l1+=", "+self.district+', ' if self.district else ""
        return l1
        
    def __eq__(self,another):
        for k,v in self.__dict__.items():
            if k not in ['variable_type','display_type'] and v != getattr(another,k,None):
                return False
        return True
        
    def __str__(self):
        return self.full_address


class Addresses(object):
    def __init__(self, address_list: List[Address]) -> None:
        self.addresses=address_list
    
    def _specific_address(self,v_type):
        address=[address for address in self.addresses if address.variable_type==v_type]
        return address[0] if address else None
    
    @property
    def mailing(self):
        return self._specific_address('mailing_address')
    
    @property
    def residential(self):
        return self._specific_address('residential_address')
    
    @property
    def business(self):
        return self._specific_address('business_address')
    
    # return the first work location
    @property
    def working(self):
        return self._specific_address('working_address')
    
    # return all work locations
    @property
    def workings(self):
        return [address for address in self.addresses if address.variable_type=='working_address']

    @property
    def PreferredAddress(self):
        for addr_type in ['business_address','residential_address','mailing_address','working_address']:
            addr=self._specific_address(addr_type)
            if addr: return addr 
    
    def getPreferredAddressFromList(self,type_list):
        for addr_type in type_list:
            addr=self._specific_address(addr_type)
            if addr: return addr
    
    




from webform.models.definition import Action
from webform.models.bcpnpmodel_reg import BcpnpModelReg, BcpnpEEModelReg
from webform.bcpnp.dashboard import DashboardReg
from model.common.language import Languages
from webform.bcpnp.data import language_map
from typing import Union


class Language:
    def __init__(self, person: Union[BcpnpModelReg, BcpnpEEModelReg]):
        self.person = person

    def fill(self):
        language = Languages(self.person.language).PreferredLanguage
        has_language_tested = [
            {
                "action_type": Action.Radio.value,
                "label": "Have you completed a valid language proficiency test within the past two years? ",
                "id": "#BCPNP_App_LangTest_Completed-Yes"
                if language
                else "#BCPNP_App_LangTest_Completed-No",
            }
        ]

        test_admin_info_head = []
        test_admin_info_foot = []

        match language.test_type:
            case "IELTS":
                test_admin_info_head = [
                    {
                        "action_type": Action.Input.value,
                        "label": "Date of administrator's signature",
                        "id": "#BCPNP_App_LangTest_Date",
                        "value": language.report_date.strftime("%Y-%m-%d"),
                        "required": True,
                        "length": 100,
                    }
                ]
                test_admin_info_foot = [
                    {
                        "action_type": Action.Input.value,
                        "label": "Test report form number",
                        "id": "#BCPNP_App_LangTest_CertNo",
                        "value": language.registration_number,
                        "required": True,
                        "length": 100,
                    }
                ]
        type_of_test = [
            {
                "action_type": Action.Select.value,
                "label": "Type of test taken",
                "id": "#BCPNP_App_LangTest_Type",
                "value": language_map[language.test_type],
            }
        ]
        language_details = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "Listening score",
                    "id": "#BCPNP_App_LangTest_ResListening",
                    "value": str(language.listening),
                    "required": True,
                    "length": 100,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Reading score",
                    "id": "#BCPNP_App_LangTest_ResReading",
                    "value": str(language.reading),
                    "required": True,
                    "length": 100,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Writing score",
                    "id": "#BCPNP_App_LangTest_ResWriting",
                    "value": str(language.writting),
                    "required": True,
                    "length": 100,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Speaking score",
                    "id": "#BCPNP_App_LangTest_ResSpeaking",
                    "value": str(language.speaking),
                    "required": True,
                    "length": 100,
                },
            ]
            if language
            else []
        )

        # dashboard = DashboardReg()
        actions = (
            # dashboard.jump("Language")
            has_language_tested
            + type_of_test
            + test_admin_info_head
            + language_details
            + test_admin_info_foot
            # + dashboard.save
        )
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Language",
                "actions": actions,
                "id": "body > div > main > div.layout-container > div > div > div > uf-form > div > ng-transclude > div > uf-tabset > div > ul > li:nth-child(6) > a",
                "next_page_tag":"#BCPNP_App_HaspaidRep-Yes2"
            }
        ]



from typing import List, Optional, Union
from model.common.address import Address
from model.common.educationbase import EducationBase
from model.common.phone import Phone
from model.common.commonmodel import CommonModel
from model.common.employmentbase import EmploymentBase
from model.common.jobofferbase import JobofferBase
from model.common.id import ID
from model.common.person import Person
from model.common.language import LanguageBase
from datetime import date, datetime
from pydantic import BaseModel, EmailStr, root_validator, validator
from model.common.utils import normalize
from webform.prportal.data.country_residence import country_residence


class PersonId(ID):
    pass


class Rcic(BaseModel):
    first_name: Optional[str]
    last_name: Optional[str]
    telephone: Optional[str]


class Language(LanguageBase):
    date: Optional[date]
    report_date: Optional[date]
    registration_number: Optional[str]
    pin: Optional[str]


class General(BaseModel):
    legal_name: str
    operating_name: Optional[str]


class ErAddress(Address):
    pass


class Joboffer(JobofferBase):
    phone: str
    is_working: bool


class Employment(EmploymentBase):
    bcpnp_qualified: bool


class Marriage(BaseModel):
    marital_status: Optional[str]
    married_date: Optional[date]
    sp_last_name: Optional[str]
    sp_first_name: Optional[str]
    sp_in_canada: Optional[bool]
    sp_canada_status: Optional[str]
    sp_canada_status_end_date: Optional[date]
    sp_in_canada_other: Optional[str]
    sp_in_canada_work: Optional[bool]
    sp_canada_occupation: Optional[str]
    sp_canada_employer: Optional[str]


class Personal(Person):
    email: EmailStr

    used_last_name: Optional[str]
    used_first_name: Optional[str]
    place_of_birth: str
    country_of_birth: str
    did_eca: bool
    eca_supplier: Optional[str]
    eca_number: Optional[str]
    ita_assessed: bool
    ita_assess_number: Optional[str]

    _normalize_used_first_name = validator(
        "used_first_name", allow_reuse=True, check_fields=False
    )(normalize)
    _normalize_used_last_name = validator(
        "used_last_name", allow_reuse=True, check_fields=False
    )(normalize)

    @property
    def user_id(self):
        dob = self.dob.strftime(("%Y-%m-%d"))
        return (
            self.last_name[0].upper()
            + self.first_name[0]
            + dob.split("-")[0]
            + dob.split("-")[1]
            + dob.split("-")[2]
        )

    @property
    def password(self):
        return "Super" + str(datetime.today().year) + "!"


class Education(EducationBase):
    city: Optional[str]
    country: Optional[str]
    is_trade: Optional[bool]


class Family(BaseModel):
    relationship: Optional[str]
    last_name: Optional[str]
    first_name: Optional[str]
    native_last_name: Optional[str]
    native_first_name: Optional[str]
    date_of_birth: Optional[date]
    date_of_death: Optional[date]
    place_of_birth: Optional[str]
    birth_country: Optional[str]
    marital_status: Optional[str]
    email: Optional[EmailStr]
    address: Optional[str]


class Bcpnp(BaseModel):
    account:Optional[str]
    password:Optional[str]
    # has_current_app: bool
    has_applied_before: bool
    pre_file_no: Optional[str]
    submission_date: Optional[date]
    case_stream: Optional[str]
    q1: bool
    q1_explaination: Optional[str]
    q2: bool
    q2_explaination: Optional[str]
    q3: bool
    q3_explaination: Optional[str]
    q4: bool
    q4_file_number: Optional[str]
    q4_explaination: Optional[str]
    q5: bool
    q5_explaination: Optional[str]
    q6: bool
    q6_explaination: Optional[str]
    q7: bool
    q7_explaination: Optional[str]


class Ee(BaseModel):
    ee_profile_no: str
    ee_expiry_date: date
    ee_jsvc: str
    ee_score: str
    ee_noc: str
    ee_job_title: str


class BcpnpModelReg(CommonModel):
    personal: Personal
    general: General
    joboffer: Joboffer
    address: List[Address]
    eraddress: List[ErAddress]
    phone: List[Phone]
    personid: List[PersonId]
    bcpnp: Bcpnp
    education: List[Education]
    employment: List[Employment]
    language: List[Language]
    rcic: Rcic
    marriage:Marriage

    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self, excels=None, output_excel_file=None):
        if output_excel_file:
            excels=self.getExcels(['excel/er.xlsx','excel/pa.xlsx','excel/bcpnp.xlsx'])
        else:
            if excels is None or len(excels) == 0:
                raise ValueError(
                    "You must input excel file list as source data for validation"
                )
        super().__init__(excels, output_excel_file, globals())


class BcpnpEEModelReg(BcpnpModelReg):
    ee: Ee

    def __init__(self, excels=None, output_excel_file=None):
        super().__init__(excels=excels, output_excel_file=output_excel_file)



from webform.models.definition import Action, Role
from webform.models.prmodel import PrModel
from typing import List


class F5562:
    def __init__(self, pa: PrModel, sp: PrModel | None, dps: List[PrModel]):
        self.pa = pa
        self.sp = sp
        self.dps = dps

    @property
    def pdf(self):
        return [{"action_type": Action.Pdf.value}]

    def didNotTravel(self, value):
        return [
            {
                "action_type": "Checkbox",
                "label": "Did not travel",
                "id": "#haveNotTravelled",
                "value": value,
            }
        ]

    def goTo5562(self):
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Start/Edit for 5562 form",
                "actions": [],
                "next_page_tag":"body > pra-root > pra-localized-app > main > div > pra-imm5562 > lib-navigation-buttons > div > button.btn.btn-primary",
                "id": "body > pra-root > pra-localized-app > main > div > pra-intake-landing-page > pra-web-form-table > div > table > tbody > tr:nth-child(22) > td:nth-child(5) > button",
            },
            {
                "action_type": Action.WebPage.value,
                "page_name": "Continue for 5562 form",
                "actions": [],
                "next_page_tag":"#haveNotTravelled",
                "id": "body > pra-root > pra-localized-app > main > div > pra-imm5562 > lib-navigation-buttons > div > button.btn.btn-primary",
            },
        ]

    def makeContentPaSp(self, role):
        value = []
        destination_id = {
            Role.PA: "#trip-destination-",
            Role.SP: "#sectionB-destination-",
        }
        person_obj = self.pa if role == Role.PA else self.sp

        for index, t in enumerate(person_obj.travel):
            block = [
                {
                    "action_type": Action.Input.value,
                    "label": "From",
                    "id": "#trip-from-" + str(index),
                    "value": t.start_date.strftime("%Y/%m"),
                    "length": 10,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "To",
                    "id": "#trip-to-" + str(index),
                    "value": t.end_date.strftime("%Y/%m"),
                    "length": 10,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Destination",
                    "id": destination_id[role] + str(index),
                    "value": t.destination,
                    "length": 100,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Length",
                    "id": "#trip-length-" + str(index),
                    "value": str(t.length),
                    "length": 100,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Purpose of travel",
                    "id": "#trip-purpose-" + str(index),
                    "value": t.purpose,
                    "length": 100,
                    "required": True,
                },
            ]
            value.append(block)
        return [
            {
                "action_type": Action.RepeatSection.value,
                "button_text": "Add another",
                "value": value,
            }
        ] + self.pdf

    def makeContentDp(self, dp: PrModel, start_index=0):
        value = []

        for index, t in enumerate(dp.travel):
            block = [
                {
                    "action_type": Action.Input.value,
                    "label": "Given name",
                    "id": "#trip-givenName-" + str(index + start_index),
                    "value": dp.personal.first_name+" "+dp.personal.native_first_name,
                    "length": 100,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "From",
                    "id": "#trip-from-" + str(index + start_index),
                    "value": t.start_date.strftime("%Y/%m"),
                    "length": 10,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "To",
                    "id": "#trip-to-" + str(index + start_index),
                    "value": t.end_date.strftime("%Y/%m"),
                    "length": 10,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Destination",
                    "id": "#sectionC-destination-" + str(index + start_index),
                    "value": t.destination,
                    "length": 100,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Length",
                    "id": "#trip-length-" + str(index + start_index),
                    "value": str(t.length),
                    "length": 100,
                    "required": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Purpose of travel",
                    "id": "#trip-purpose-" + str(index + start_index),
                    "value": t.purpose,
                    "length": 100,
                    "required": True,
                },
            ]
            value.append(block)
        return value

    def makeFormDps(self):
        start_index = 0
        values = []
        for dp in self.dps:
            values += self.makeContentDp(dp, start_index)
            start_index += len(dp.travel)

        actions = (
            self.didNotTravel(False)
            + [
                {
                    "action_type": Action.RepeatSection.value,
                    "button_text": "Add another",
                    "value": values,
                },
            ]
            + self.pdf
            if len(values) > 0
            else self.didNotTravel(True) + self.pdf
        )
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Imm5562 page 4 for dependants",
                "actions": actions,
                "next_page_tag":"body > pra-root > pra-localized-app > main > div > pra-intake-landing-page > div.intake-landing-page_submit-application.ng-star-inserted > button",
                "id": "body > pra-root > pra-localized-app > main > div > pra-imm5562 > lib-navigation-buttons > div > button.btn.btn-primary",
            }
        ]

    def makeFormPaSp(self):
        # make pa form
        pa = [
            {
                "action_type": Action.Input.value,
                "label": "Your family name",
                "value": self.pa.personal.last_name+"  "+self.pa.personal.native_last_name,
                "id": "#familyName",
                "length": 100,
                "required": True,
            },
            {
                "action_type": Action.Input.value,
                "label": "Your given name",
                "value": self.pa.personal.first_name+" "+self.pa.personal.native_first_name,
                "id": "#givenName",
                "length": 100,
                "required": True,
            },
        ]
        if len(self.pa.travel) > 0:
            pa_travel_content = (
                self.didNotTravel(False) + pa + self.makeContentPaSp(Role.PA)
            )
        else:
            pa_travel_content = self.didNotTravel(True)
        # make sp form
        if self.sp and len(self.sp.travel) > 0:
            sp_travel_content = self.didNotTravel(False) + self.makeContentPaSp(Role.SP)
        else:
            sp_travel_content = self.didNotTravel(True)

        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Imm5562 page 2 for principal applicant",
                "actions": pa_travel_content,
                "next_page_tag":"#haveNotTravelled",
                "id": "body > pra-root > pra-localized-app > main > div > pra-imm5562 > lib-navigation-buttons > div > button.btn.btn-primary",
            },
            {
                "action_type": Action.WebPage.value,
                "page_name": "Imm5562 page 3 for spouse",
                "actions": sp_travel_content,
                "next_page_tag":"#haveNotTravelled",
                "id": "body > pra-root > pra-localized-app > main > div > pra-imm5562 > lib-navigation-buttons > div > button.btn.btn-primary",
            },
        ]

    def fill(self):
        return self.goTo5562() + self.makeFormPaSp() + self.makeFormDps()



import xml.etree.ElementTree as ET
from source.tablesheet import TableNode
from source.excel import Excel
from model.common.convert import NameCodeConvert
from model.data.country import country
from model.data.language import language
from model.data.other import phone_type,tr_canada_status,tr_marital_status,tr_form_study_level,tr_form_study_field,tr_portal_study_level,tr_portal_study_field,marital_stauts_5645
from model.data.city import canada_city,canada_province

class PdfFormModel():
    """
    This model is for get information for TR vis or permit pdf xml, and save it to xlsx
    """
    def __init__(self,xml_file,model) -> None:
        self.data_model=model.get('data_model')
        self.convert_model=model.get('convert_model')
        self.remove_model=model.get('remove_model')
        self.tree=ET.ElementTree(file=xml_file)
        self.root=self.tree.getroot()
        self.data={}    # data from pdf xml, and then foe excel file after some operations
        self._getData() # get raw data from pdf xml    
        self._convert() # convert country, language, etc...
        self._assembl() # use temp variables to assemble excel needed variables
        self._remove() # remove not needed variables
    
    def _getSubItem(self,element,variable_xpath_pairs):
        temp_dict={}
        for variable,xpath in variable_xpath_pairs.items():
            try:
                temp_dict[variable]=element.find(xpath).text
            except:
                print(f"{xpath} is invalid xpath")
        return temp_dict

    def _getData(self):
        for sheet,source_pair in self.data_model.items():
            if sheet not in ['occupation','children','siblings']: 
                self.data[sheet]=self._getSubItem(self.root,source_pair)
            else:
                self._getSpecial()
            
    
    def _getSpecial(self):
        pass

    # convert some items to excel format 
    def _execConvert(self,pairs:dict,convert_item):
        for sheet, items in pairs.items():
            for item in items:
                # change value dict with converted country name
                sheet_data = self.data.get(sheet)
                if isinstance(sheet_data,list):
                    for i,sheet_item in enumerate(sheet_data):
                        code=sheet_data and sheet_data[i].get(item)
                        if code: self.data[sheet][i][item]=NameCodeConvert(convert_item).getName(code)
                else:
                    code=sheet_data and sheet_data.get(item)
                    if code: self.data[sheet][item]=NameCodeConvert(convert_item).getName(code)
        
    def _convert(self):
        global_variables=globals()
        for pairs, model in self.convert_model.items():
            model = global_variables[pairs[:-6]]  # get model by pairs name remove '_pairs'
            self._execConvert(self.convert_model[pairs], model)

        canada_city_pairs = {
            'sp': ["city"],
            'wp': ["employment_city"],
            'visa': ["city"],
            'vrincanada': ["city"],
            'spincanada': ["city"],
            'wpincanada': ["work_city"],
        }
        self._convert_city(canada_city_pairs, canada_city)
    
    def _convert_city(self,canada_city_pairs,canada_city):
        the_cities={}
        for cities in canada_city.values():
            for city, code in cities.items():
                the_cities[city]=code
        self._execConvert(canada_city_pairs,the_cities)
        
    def _assembl(self):
        # for English or French
        self.data['personal']['which_one_better']='English' if self.data['personal']['which_one_better']=='01' else "French"
        trcase_trcasein='trcase' if 'trcase' in self.data.keys() else 'trcasein'
        self.data[trcase_trcasein]['service_in']='English' if self.data[trcase_trcasein]['service_in']=='01' else "French"
        
        # assembly dob 
        self.data['personal']['dob']=self.data['personal'].get('dob_year')+"-"+self.data['personal'].get('dob_month')+"-"+self.data['personal'].get('dob_day')
        dob_y,dob_m,dob_d=self.data['marriage'].get('pre_sp_dob_year'),self.data['marriage'].get('pre_sp_dob_month'),self.data['marriage'].get('pre_sp_dob_day')
        self.data['marriage']['pre_sp_dob']=dob_y+"-"+dob_m+"-"+dob_d  if all([dob_y,dob_m,dob_d]) else None
        
        self._cor_assemble()
        self._id_assemble()
        self._address_assemble()
        self._phone_assemble()
        # self._work_permit_assemble()
        self._education_assemble()
        self._occupation_assemble()
        
    # countries of residence
    def _cor_assemble(self):
        cor_dict=self.data['cor']
        cor_current={
            'country':cor_dict['current_cor_country'],
            'status':cor_dict['current_cor_status'],
            'other':cor_dict['current_cor_other'],
            'start_date':cor_dict['current_cor_start_date'],
            'end_date':cor_dict['current_cor_end_date']
            }
        cor_pre1={
            'country':cor_dict['previous_cor_country1'],
            'status':cor_dict['previous_cor_status1'],
            'other':cor_dict['previous_cor_other1'],
            'start_date':cor_dict['previous_cor_start_date1'],
            'end_date':cor_dict['previous_cor_end_date1']
            }
        cor_pre2={
            'country':cor_dict['previous_cor_country2'],
            'status':cor_dict['previous_cor_status2'],
            'other':cor_dict['previous_cor_other2'],
            'start_date':cor_dict['previous_cor_start_date2'],
            'end_date':cor_dict['previous_cor_end_date2']
            }
        # re-sort the order of cor 
        cor=[]
        for c in [cor_current,cor_pre1,cor_pre2]:
            d={}
            for key in ['start_date','end_date','country','status']:
                d[key]=c[key]
            cor.append(d)
        self.data['cor']=cor

    # ids 
    def _id_assemble(self):
        passport={'variable_type':'passport',**self.data['passport']}
        nid={'variable_type':'id',**self.data['national_id']}
        us_pr={
            'variable_type':'pr',
            'number':self.data['us_pr']['number'],
            'expiry_date':self.data['us_pr']['expiry_date']
            }
        self.data['personid']=[passport,nid,us_pr]
        [self.data.pop(x) for x in ['passport','national_id','us_pr']]

    # address 
    def _address_assemble(self):
        m_dict=self.data['mailing_address']
        r_dict=self.data['residential_address']
        m={'variable_type':'mailing_address',**m_dict}
        if r_dict['same_as_mailing']=='Y':
            r=m
        else:
            r={'variable_type':"residential_address",**r_dict}
        self.data['address']=[m,r]
        [self.data.pop(x) for x in ['mailing_address','residential_address']]
        # convert country and province
        for i,addr in enumerate(self.data['address']):
            self.data['address'][i]['country']=NameCodeConvert(country).getName(addr['country']) 
            self.data['address'][i]['province']=NameCodeConvert(canada_province).getName(addr['province']) 
    
     # phone 
    def _phone_assemble(self):
        temp_phone=[]
        for p_type in ['phone','altphone','fax']:
            d=self.data[p_type]
            if d.get('number'): #如果没有号码，就位空
                d['variable_type']='fax' if not d.get('variable_type') else d['variable_type']
                temp_phone.append({ k:v for k,v in d.items() if k in ['variable_type','country_code','number','ext']})
        [self.data.pop(x) for x in ['phone','altphone','fax']]
        self.data['phone']=temp_phone        

     # work permit TODO: 这个不应该在里面，应该是1295的特别的。 
    def _work_permit_assemble(self):
        temp_wp={}
        for d in [self.data['detailed_work'],self.data['intended_location'],self.data['intended_job']]:
            temp_wp={**temp_wp,**{ k:v for k,v in d.items()}}
        [self.data.pop(x) for x in ['detailed_work','intended_location','intended_job']]
        self.data['wp']=temp_wp
        
    # education
    def _education_assemble(self):
        self.data['education']['start_date']=self.data['education']['start_date_year']+'-'+self.data['education']['start_date_month']+'-01'
        self.data['education']['end_date']=self.data['education']['end_date_year']+'-'+self.data['education']['end_date_month']+'-01'
        # re-sort education's order
        temp={}
        for k in ['start_date','end_date','school_name','education_level','field_of_study','city','province','country']:
            temp[k]=self.data.get('education').get(k,None)
        self.data['education']=temp

    def _remove(self):
        for k, v in self.remove_model.items():
            [self.data[k].pop(item) for item in v]
            
    # occupation
    def _occupation_assemble(self):
        temp=[]
        for occupation in self.data['occupations']:
            occupation['start_date']=occupation['start_date_year']+'-'+occupation['start_date_month']+'-01' if all([occupation['start_date_year'],occupation['start_date_month']]) else ""
            occupation['end_date']=occupation['end_date_year']+'-'+occupation['end_date_month']+'-01' if all([occupation['end_date_year'],occupation['end_date_month']]) else ""
            occupation['country']=NameCodeConvert(country).getName(occupation['country']) # convert country code to country name
            occupation['province']=NameCodeConvert(canada_province).getName(occupation['province']) # convert province code to province name
            temp.append(occupation)
        temp1=[]
        for t in temp:
            d={}
            for k in ['start_date','end_date','job_title','company','city','province','country']:
                d[k]=t[k]
            temp1.append(d) 
            
        self.data['employment']=temp1    
    
    # get same xx_type
    def _getTableNodeWithType(self,table_node_list,the_type):
        try:
            for node in table_node_list:
                if node["variable_type"]==the_type:
                    return node
        except:
            return None
    
    # excel_file should be an existed excel file matching the 1294 program
    def _makeExcel(self,excel_file,protection):
        
        excel_obj=Excel(excel_file) #if path.isfile(excel_file) else self.new_excel_obj

        for sheet_name, sheet_dict in self.data.items():
            # 如果不是sheet 下一个
            if sheet_name not in excel_obj.sheets.keys():
                continue
            
            for key,value in sheet_dict.items():
                # 如果excel表中有对应的sheet和字段
                if key in excel_obj.sheets[sheet_name].data.keys():     
                    excel_obj.sheets[sheet_name].data[key].value=value
        
        for table_name,table_obj in excel_obj.tables.items():
            # this three tables will be processed independently
            if (table_name in ['personid','address','phone']):
                for index,value in enumerate(table_obj.data):
                    if value:
                        node=self._getTableNodeWithType(self.data.get(table_name),value.variable_type)
                        for k in value.__dict__.keys():
                            value=node.get(k) if node else None
                            if k not in ['variable_type','display_type']: setattr(excel_obj.tables[table_name].data[index],k,value)
            # others are normal                
            else:
                # 现在在每张表里，取出sel.data[table_name]的table_node数据字典，做成tablenode,放到表里的data中。
                node_data=self.data.get(table_name)
                if isinstance(node_data,dict):
                    excel_obj.tables[table_name].data.append(TableNode(**node_data))
                elif isinstance(node_data,list):
                    for node in node_data:
                        excel_obj.tables[table_name].data.append(TableNode(**node))
                else:
                    pass
        # 根据最终更新了variables的sheets和talbes，生成新的excel文件
        excel_obj.makeExcel(excel_file,protection=protection)








from cProfile import label
from enum import Enum
from pydantic import BaseModel
from abc import ABC,abstractmethod
from typing import Union,Optional
from models.definition import Action
from dotenv import dotenv_values


class Element(ABC,BaseModel):
    action_type:str
    id:Optional[str]
    
    @abstractmethod
    def make(self):
        pass

class InputElement(Element):
    label:str
    value:str
    length:Optional[int]
    required:Optional[bool]
    disabled:Optional[bool]
    
    def make(self):
        return {
            'action_type':self.action_type,
            "label":self.label,
            "id":self.id,
            "value":self.value,
            "length":self.length,
            "required":self.required,
            "disabled":self.disabled
        }


class AreatextElement(InputElement):
    pass

class SelectElement(Element):
    label:str
    value:str
    
    def make(self):
        return {
            'action_type':self.action_type,
            "label":self.label,
            "id":self.id,
            "value":self.value
        }

class CheckboxElement(Element):
    label:str
    def make(self):
        return {
            'action_type':self.action_type,
            'label':self.label,
            'id':self.id
        }

# Radio element 
# class RadioElement(Element):
#     group_label:str
#     id:str
#     id_label:str
#     value: bool
#     children:Union[list,None]


# Yes/No radio element
class RadioYesNoElement(Element):
    yes_id:str
    no_id:str
    value: bool
    def make(self):
        return {
            "action_type":self.action_type,
            "id": self.yes_id if self.value else self.no_id,
            "label":"Yes" if self.value else "No"
        }

# Yes/No/null radio element
class RadioYesNoNullElement(Element):
    yes_id:str
    no_id:str
    null_id:str
    value: Union[bool,None]
    def make(self):
        match self.value:
            case True:
                id_label="Yes";
                new_id=self.yes_id;
            case False:
                id_label="No";
                new_id=self.no_id;
            case None:
                id_label="Not Applicable";
                new_id=self.null_id;
        return {
            "action_type":self.action_type,
            "id":new_id,
            "label":id_label,
        }

class ButtonElement(Element):
    label:str
    id:str
    # children:Union[list,None]
    
    def make(self):
        return {
            'action_type':self.action_type,
            "label":self.label,
            "id":self.id
        }

class TurnPage(Element):
    label:str
    def make(self):
        return {
            "action_type":Action.Turnpage.value,
            "label":self.label,
            "id":self.id
    }


class LoginElement(Element):
    rcic:str
    portal:str
    account_element_id:str
    password_element_id: str
    login_button_id:str
    success_element_id:str

    def make(self):
        config = dotenv_values(".env")
        account_key=self.rcic.upper()+"_"+self.portal.upper()+"_ACCOUNT"
        password_key=self.rcic.upper()+"_"+self.portal.upper()+"_PASSWORD"
        return {
            'action_type':self.action_type,
            "account": config.get(account_key),
            "password": config.get(password_key),   #TODO: 后续需要考虑加密，然后在js中解密
            "account_element_id":self.account_element_id,
            "password_element_id": self.password_element_id,
            "login_button_id":self.login_button_id
        }

class SecurityElement(Element):
    question_element_id: str
    answer_element_id: str
    continue_element_id: str
    success_element_id: str
    security_answers:dict
    
    def make(self):
        return {
            'action_type':self.action_type,
            "question_element_id":self.question_element_id,
            "answer_element_id": self.answer_element_id,
            "continue_element_id": self.continue_element_id,
            "success_element_id": self.success_element_id,
            "security_answers":self.security_answers
        }

class DependantSelectElement(Element):
    select1:SelectElement
    select2:SelectElement
    def make(self):
        return {
            "action_type":self.action_type,
            "select1":self.select1,
            "select2":self.select2
        }



import shutil
import os
from socket import timeout
import time
import argparse

from pywinauto.application import Application
from pywinauto.keyboard import send_keys

from pdfform import builders
from pdfform import loaders
from pdfform import config

import os, sys

# Get project's home directory,
BASEDIR = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
# All data directory
DATADIR = os.path.abspath(os.path.join(BASEDIR, "data"))


def check_env():
    if not os.path.isfile(config.ACROBAT_READER_PATH):
        print("Acrobat reader not found. Please install it first.")
        exit()
    if not os.path.isdir(config.OUTPUT_PATH):
        print("Output directory doesn't exist. Please check.")
        exit()


def prepare_document(form_type: str, name: str):
    """Create a new empty pdf file to fill"""

    # copy empty template file to output folder
    # dirname = os.path.dirname(os.path.abspath(__file__))
    src = DATADIR + f"/forms/{form_type}_empty.pdf"
    dst = config.OUTPUT_PATH + name + ".pdf"
    shutil.copy2(src, dst)

    # open file with acrobat reader
    app = Application(backend="uia").start(config.ACROBAT_READER_PATH + " " + dst)
    # TODO: find better way to detect ready
    time.sleep(10)  # wait 5 seconds for file open finished.
    # dlg_spec = app.window(title="Desktoptest.pdf(SECURED)-AdobeAcrobatReaderDC(64-bit)")
    # dlg_spec.wait("ready", timeout=30)
    # app.Properties.print_control_identifiers()


def finish_document():
    """Validate, save and close document"""

    send_keys("^S")  # Ctrl + S to save
    send_keys("^W")  # Ctrl + W to close


def get_args():
    """Get command-line arguments"""

    parser = argparse.ArgumentParser(
        description="PDF Filler -- automatically fill application forms.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    )

    parser.add_argument(
        "form_name",
        help="PDF form type: currently supported form [1295, 5708, 5709, 5710]",
        default="demo",
    )

    parser.add_argument(
        "-f",
        "--file",
        help="Data file used to fill form. If no file specified, the tool will use example data to fill for demo purpose.",
        metavar="data_file_name",
        type=argparse.FileType("r"),
        default=None,
    )

    parser.add_argument(
        "-o",
        "--output",
        help="Output file name. If not specified, file name will be test.pdf.",
        metavar="output_file_name",
        default="test",
    )

    parser.add_argument(
        "-v", "--verbose", help="verbose output filling steps", action="store_true"
    )

    return parser.parse_args()


def main():
    """main function"""

    args = get_args()
    form_name = args.form_name
    in_file = args.file
    verbose = args.verbose
    out_file = "test"

    check_env()

    if form_name not in ["1295", "5708", "5709", "5710"]:
        print(f'Error: form "{form_name}" is not supported.')
        quit()

    applicant = loaders.load_form_data(in_file, form_name)
    prepare_document(form_name, out_file)
    form = builders.build_form(form_name, applicant)
    form.fill_form(0, verbose)
    finish_document()


if __name__ == "__main__":
    main()



from datetime import datetime
from termcolor import colored
from utils import utils
from engine.scoringbc import ScoringBCPNP
import argparse

bcpnp_stream={
    'SW':"Skill_Worker",
    'IG':"International_Graduate",
    'ELSS':'Entry_Level',
    'EESW':'EE_Skill_Worker',
    'EEIG':'EE_International_Graduate'
}
bcpnp_education = {
    0:'Doctor',
    1:'Master',
    2:'Bachelor',
    3:'Post Graduate Certificate or Diploma',
    4:'Trades certification',
    5:'Associate Degree',
    6:'Non-trades certification or Diploma',
    7:'Secondary'
}
bcpnp_education_bonus = {
    0:'BC Post-secondary',
    1:'Canada Post-secondary',  # Outside of BC
    2:'ECA',  # with Education Credential Assessment
    # Successfully completed the Industry Training Authority British Columbia (ITABCs) challenge certification process
    3:'Trade certificate',
    4:'Nothing'
}

bcpnp_region = {
        0:'Stikine',
        1:'Central Coast',
        2:'Northern Rockies',
        3:'Mount Waddington',
        4:'Skeena-Queen Charlotte',
        5:'Powell River',
        6:'Sunshine Coast',
        7:'Kootenay-Boundary',
        8:'Alberni-Clayoquot',
        9:'Kitimat-Stikine',
        10:'Bulkley-Nechako',
        11:'Squamish-Lillooet',
        12:'Strathcona',
        13:'Columbia-Shuswap',
        14:'East Kootenay',
        15:'Peace River',
        16:'Comox Valley',
        17:'Cariboo',
        18:'Central Kootenay',
        19:'Okanagan-Similkameen',
        20:'Cowichan Valley',
        21:'North Okanagan',
        22:'Fraser-Fort George',
        23:'Thompson-Nicola',
        24:'Nanaimo',
        25:'Central Okanagan',
        26:'Capital',
        27:'Fraser Valley',
        28:'Greater Vancouver'
    }

def getParams(args):
    # today=datetime.today()
    # enddate=str(today.year)+'-'+str(today.month)+'-'+str(today.day)
    # startdate=str(today.year-1)+'-'+str(today.month)+'-'+str(today.day)

    oneyear=",".join(args.oneyear or ['0'])
    isworking=','.join(args.isworking or ['0'])
    hourly_rate=",".join(args.rate or ['25'])
    working_hours=",".join(args.hours or ['40'])
    clbs=",".join(args.language or ['5']) 

    stream=bcpnp_stream.get(args.stream,'Skill_Worker') if args.stream else 'Skill_Worker'
    education=bcpnp_education.get(int(args.education),'Bachelor') if args.education else "Bachelor"
    education_bonus=bcpnp_education_bonus.get(int(args.bonus),'ECA') if args.bonus else 'ECA'
    region=bcpnp_region.get(int(args.area),'Greater Vancouver') if args.area else 'Greater Vancouver'
    work_experience=int(args.experience or 0)*12
    params={
        # 'stream':stream,
        'noc':args.noc or '1111',
        'work_experience':work_experience,
        'education':education,
        'education_bonus':education_bonus,
        'has_one_year_canadian_experience':oneyear,
        'is_working_in_the_position':isworking,
        'hourly_rate':hourly_rate,
        'working_hours':working_hours,
        'region':region,
        'clbs':clbs
        
        # 'start_date':args.startdate or startdate,
        # 'end_date':args.enddate or enddate
        }
    return params


def output(data):
    newlist=[]
    for r in list(zip(data['reports'],data['solutions'])):
        unit={**r[0],**r[1]}
        newlist.append(unit)
    newlist.sort(key=lambda x:x.get('total_points'),reverse=True)
    print(colored(f'Totally {len(newlist)} solutions','green'))
    utils.printListDictFields(
        newlist,
        fields=['total_points','clb','hourly_rate','working_hours','is_working_in_the_position','has_one_year_canadian_experience','invited_times','percentage'],
        titles=['Points','CLB','Rate','Hours','Working','One_Year','ITA_Times',"Percentage"]
        )
    print(colored(f'Client facts:','green'))
    record=newlist[0]
    client_facts=[
        ['Education','Bonus','Experience','NOC','Tech Pilot','Region'],
        [record['education'],record['education_bonus'],record['work_experience'],record['noc'],record['tech_pilot'],record['region']]
    ]
    utils.printFList2D(client_facts)

    # ITA analysis 
    # print(colored(f'ITA summary:','green'))
    # record=newlist[0]
    # ita_summary=[
    #     ['From','To','Stream','Total Rounds','Minimum Points','Average Points','Maximum Points'],
    #     [record['start_date'],record['end_date'],record['stream'],record['total_rounds'],record['ita_min'],record['ita_avg'],record['ita_max']]
    # ]
    # utils.printFList2D(ita_summary)


def main():
    parser=argparse.ArgumentParser(description="used for calculating BCPNP Skills Iimmigration points")
    parser.add_argument("-n", "--noc", help="input noc code")
    parser.add_argument("-w", "--isworking", nargs='+',help="adding -w plus 0 1, means is working on the position or not")
    parser.add_argument("-o", "--oneyear",nargs='+', help="adding -o plus 0 1, means with one year direct Canadian experience or not")
    parser.add_argument("-r", "--rate", nargs='+',help="input hourly rates, exmp: 25 26...")
    parser.add_argument("-t", "--hours",nargs='+', help="input working hours per week, exmp: 30 40")
    parser.add_argument("-we", "--experience", help="input direct related work experience years")
    parser.add_argument("-a", "--area", help="optional,default is 28 (Greater Vancouver) input area index")
    parser.add_argument("-e", "--education", help="input educaton index")
    parser.add_argument("-b", "--bonus", help="input education bonus index")
    parser.add_argument("-l", "--language", nargs='+',help="input language clb level")
    parser.add_argument("-i","--information",help="get indexes for area, education, and education bonus",action='store_true')
    parser.add_argument("-sd", "--startdate", help="input begining date. format: 2020-01-01")
    parser.add_argument("-ed", "--enddate", help="input ending date. format: 2020-01-01,Default is today")
    parser.add_argument("-s", "--stream", help="input the stream: SW,IG,ELSS,EESW,EEIG")

    args = parser.parse_args()
    if args.information:
        print(colored('Eduation levels','green'))
        utils.printFDict(bcpnp_education)
        print(colored('Eduation Bonus','green'))
        utils.printFDict(bcpnp_education_bonus)
        print(colored('Regions','green'))
        utils.printFDict(bcpnp_region)
        return 0

    params=getParams(args)
    print(params)
    from copy import deepcopy
    bcsp_data=deepcopy(params)
    for is_working in params['is_working_in_the_position'].split(','):
        for one_year_exp in params['has_one_year_canadian_experience'].split(','):
            for wh in params['working_hours'].split(','):
                for hr in params['hourly_rate'].split(','):
                    for clb in params['clbs'].split(','):
                        bcsp_data['clb']=clb
                        bcsp_data['hourly_rate']=hr
                        bcsp_data['has_one_year_canadian_experience']=bool(one_year_exp)
                        bcsp_data['is_working_in_the_position']=bool(is_working)
                        bcsp_data['working_hours']=float(wh)
                        data=ScoringBCPNP(**bcsp_data)
                        print(data.total_point)
                        # print(data.noc_level_point,'noc level point')
                        # print(data.noc_00_point,'noc_00 point')
                        # print(data.wage_point,'wage_points')
                        # print(data.work_experience_point,'work_experience_point')
                        # print(data.working_in_the_position_point,'working_in_the_position_point')
                        # print(data.one_year_Canadian_work_experience_point,'one_year_Canadian_work_experience')
                        # print(data.educaiton_level_point,'educaiton_level_points')
                        # print(data.education_bonus_point,'education_bonus_points')
                        # print(data.region_point,'region_points')
                        # print(data.language_level_point,'language_level_points')
    # output(data)
   

if __name__=='__main__':
    main()






from webform.models.definition import Action
from webform.models.bcpnpmodel_reg import BcpnpModelReg, BcpnpEEModelReg
from webform.models.bcpnpmodel_app import BcpnpModelApp, BcpnpEEModelApp
from webform.bcpnp.dashboard import DashboardApp, DashboardReg
from typing import Union


class Submit:
    def __init__(
        self,
        person: Union[BcpnpModelReg, BcpnpEEModelReg, BcpnpModelApp, BcpnpEEModelApp],
    ):
        self.person = person
        self.is_reg = (
            True if type(self.person) in [BcpnpModelReg, BcpnpEEModelReg] else False
        )

    def fill(self):
        check_sign = [
            {
                "action_type": Action.Checkbox.value,
                "label": "Check consent box",
                "id": "body > div > main > div.layout-container > div > div > div > uf-form > div > ng-transclude > div > uf-tabset > div > div.tab-content > ng-transclude > uf-tab:nth-child(6) > div > uf-panel:nth-child(2) > div > uf-panel-body > div > uf-agreement > div > div:nth-child(1) > div.agreement-checkbox-container > div > label"
                if self.is_reg
                else "body > div > main > div.layout-container > div > div > div > uf-form > div > ng-transclude > div > uf-tabset > div > div.tab-content > ng-transclude > uf-tab:nth-child(8) > div > uf-panel:nth-child(2) > div > uf-panel-body > div > div > uf-agreement > div > div:nth-child(1) > div.agreement-checkbox-container > div > label > input",
                "value": True,
            },
            {
                "action_type": Action.Input.value,
                "label": "Registrant full name",
                "id": "#BCPNP_App_ConsentRegName",
                "value": self.person.personal.full_name,
                "required": True,
                "length": 100,
            },
        ]
        spouse_check_sign = (
            [
                {
                    "action_type": Action.Checkbox.value,
                    "label": "Check consent box",
                    "id": "body > div > main > div.layout-container > div > div > div > uf-form > div > ng-transclude > div > uf-tabset > div > div.tab-content > ng-transclude > uf-tab:nth-child(6) > div > uf-panel:nth-child(2) > div > uf-panel-body > div > uf-agreement > div > div:nth-child(1) > div.agreement-checkbox-container > div > label"
                    if self.is_reg
                    else "body > div > main > div.layout-container > div > div > div > uf-form > div > ng-transclude > div > uf-tabset > div > div.tab-content > ng-transclude > uf-tab:nth-child(8) > div > uf-panel:nth-child(2) > div > uf-panel-body > div > div > div > uf-agreement > div > div:nth-child(1) > div.agreement-checkbox-container > div > label > input",
                    "value": True,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Registrant full name",
                    "id": "#BCPNP_App_ConsentSpouseName",
                    "value": self.person.marriage.sp_first_name
                    + " "
                    + self.person.marriage.sp_last_name,
                    "required": True,
                    "length": 100,
                },
            ]
            if self.person.marriage.marital_status in ["Married", "Common-Law"] and type(self.person) in [BcpnpModelApp, BcpnpEEModelApp]
            else []
        )

        has_rep = [
            {
                "action_type": Action.Radio.value,
                "label": "Did you hair a pair Rep?",
                "id": "#BCPNP_App_HaspaidRep-Yes1"
                if self.person.rcic.first_name
                else "#BCPNP_App_HaspaidRep-Yes2",
            },
        ]

        rep_details = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "Representative Family Name(s)",
                    "id": "#BCPNP_App_RepFamilyName",
                    "value": self.person.rcic.last_name,
                    "required": True,
                    "length": 100,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Representative Given Name(s)",
                    "id": "#BCPNP_App_RepGivenName",
                    "value": self.person.rcic.first_name,
                    "required": True,
                    "length": 100,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Phone number of paid representative",
                    "id": "#BCPNP_App_RepPhone",
                    "value": self.person.rcic.telephone,
                    "required": True,
                    "length": 100,
                },
            ]
            if self.person.rcic.first_name
            else []
        )
        wait = [{"action_type": Action.Wait.value, "duration": 2000}]

        dashboard = DashboardReg() if self.is_reg else DashboardApp()
        read_agree = (
            [
                {
                    "action_type": Action.Checkbox.value,
                    "label": "I have read and agree",
                    "id": "#BCPNP_PaymentRefundPolicy",
                    "value": True,
                }
            ]
            if type(self.person) in [BcpnpModelApp, BcpnpEEModelApp]
            else []
        )
        actions = (
            # dashboard.jump("Submit")
            check_sign
            + spouse_check_sign
            + has_rep
            + rep_details
            + read_agree
            + dashboard.save
            + wait
        )
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Submit",
                "actions": actions,
                "id": None,
            }
        ]



from .context import DATADIR
from typing import List
from model.common.commonmodel import CommonModel
from model.recruit.jobofferdata import JobOffer,General,ErAddress,ErAddresses
from model.common.address import Address,Addresses
from model.common.person import Person
from model.common.wordmaker import WordMaker
import os

class Personal(Person):
    def __str__(self):
        return self.full_name
        
class JobofferModel(CommonModel):
    general:General
    joboffer:JobOffer
    eraddress:List[ErAddress]
    personal:Personal
    address:List[Address]
    
    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self,excels=None,output_excel_file=None):
        if output_excel_file:
            excels=self.getExcels(['excel/pa.xlsx','excel/er.xlsx'])
        else:
            if excels is None and len(excels)==0:
                raise ValueError('You must input excel file list as source data for validation')
        # call parent class for validating
        super().__init__(excels,output_excel_file,globals())
        
class JobofferModelDocxAdapater():
    """This is an adapater to bridging job ad model data and docx data
    """

    def __init__(self,joboffer_obj: JobofferModel):
        # get original obj, which will be used to generate some value based on it's object methods. 
        # 此处用来处理list里面的一些内容。 
        self.joboffer_obj=joboffer_obj
        eraddresses=ErAddresses(self.joboffer_obj.eraddress)
        self.joboffer_obj.eraddress=eraddresses.working
        
        addresses=Addresses(self.joboffer_obj.address)
        self.joboffer_obj.address=addresses.residential
        
    def make(self,output_docx,template_no=None):
        file_name="word/joboffer"+str(template_no)+".docx"     
        template_path=os.path.abspath(os.path.join(DATADIR,file_name))        
        wm=WordMaker(template_path,self.joboffer_obj,output_docx)
        wm.make()
    




from pdfform import context
from datetime import date
from typing import Union

from .application_form import ApplicationForm
from . import option_lists

from model.common.cor import CORs
from model.common.id import ID, IDs
from model.common.address import Address, Addresses
from model.common.phone import Phone, Phones
from model.common.educationbase import EducationBase, EducationHistory
from model.tr.m5710 import TrBackground


def special_YMD_handle(the_date, form):
    the_date = the_date.strftime("%Y-%m-%d") if type(the_date) == date else the_date
    year, month, day = the_date.split("-")
    form.add_text(year)
    form.add_text(month)
    form.add_text(day)


def add_header(form: ApplicationForm, form_type: str = "1295"):
    """Add header section items"""
    personal = form.applicant.personal

    form.add_text(personal.uci)
    if form_type == "1295":
        form.add_dropdown(
            form.applicant.trcase.service_in, option_lists.service_language
        )
    else:
        form.add_dropdown(
            form.applicant.trcasein.service_in, option_lists.service_language
        )

    if form_type == "5708":
        if (
            form.applicant.vrincanada.application_purpose
            == "apply or extend visitor record"
        ):
            form.add_checkbox(True)
        else:
            form.add_skip(1)
        if form.applicant.vrincanada.application_purpose == "restore status as visotor":
            form.add_checkbox(True)
        else:
            form.add_skip(1)
        if form.applicant.vrincanada.application_purpose == "TRP":
            form.add_checkbox(True)
        else:
            form.add_skip(1)
    elif form_type == "5709":
        if (
            form.applicant.spincanada.application_purpose
            == "apply or extend study permit"
        ):
            form.add_checkbox(True)
        else:
            form.add_skip(1)
        if form.applicant.spincanada.application_purpose == "restore status as student":
            form.add_checkbox(True)
        else:
            form.add_skip(1)
        if form.applicant.spincanada.application_purpose == "TRP":
            form.add_checkbox(True)
        else:
            form.add_skip(1)
    elif form_type == "5710":
        # apply WP for same employer, apply WP for new employer,restore status as worker,, TRP with new employer
        if form.applicant.wpincanada.application_purpose == "restore status as worker":
            form.add_checkbox(True)  # pick apply for wp with same employer
            form.add_skip(1)
            form.add_checkbox(True)  # pick restore status
            form.add_skip(1)
        elif form.applicant.wpincanada.application_purpose == "TRP with same employer":
            form.add_checkbox(True)  # pick apply for wp with same employer
            form.add_skip(2)
            form.add_checkbox(True)  # pick restore status
        elif form.applicant.wpincanada.application_purpose == "TRP with new employer":
            form.add_skip(1)
            form.add_checkbox(True)  # pick apply for wp with same employer
            form.add_skip(1)
            form.add_checkbox(True)  # pick restore status
        else:
            if (
                form.applicant.wpincanada.application_purpose
                == "apply WP for same employer"
            ):
                form.add_checkbox(True)
                form.add_skip(3)
            else:
                form.add_skip(1)
            if (
                form.applicant.wpincanada.application_purpose
                == "apply WP for new employer"
            ):
                form.add_checkbox(True)
                form.add_skip(2)


def add_personal_detail(form: ApplicationForm, form_type: str = ""):
    """Add personal detail items
    1. full name
    2. other name
    3. sex
    4. date of birth
    5. place of birth
    6. citizenship
    7. current country or territory of residence
    8. previous countries or territories of residence
    9. country or territory where applying
    10. current marital status
    11. previous mariage
    """
    personal = form.applicant.personal
    form.add_info("personal detail section")

    form.add_text(personal.last_name)
    form.add_text(personal.first_name)

    if personal.used_first_name:
        form.add_radio(True)
        form.add_text(personal.used_last_name)
        form.add_text(personal.used_first_name)
    else:
        form.add_radio(False)
        form.add_skip(2)

    form.add_dropdown(personal.sex, option_lists.gender)

    form.add_date(personal.dob)

    form.add_text(personal.place_of_birth)
    form.add_dropdown(personal.country_of_birth, option_lists.country_of_birth)

    form.add_dropdown(personal.citizen, option_lists.country_of_citizen)

    # residence
    residence = CORs(form.applicant.cor).current
    form.add_dropdown(residence.country, option_lists.country_of_residence)
    status = residence.status
    form.add_dropdown(status, option_lists.immigration_status)
    if status.lower() in ["visitor", "worker", "student"]:
        form.add_skip(1)
        form.add_date(residence.start_date)
        form.add_date(residence.end_date)
    elif status.lower() in ["other"]:
        form.add_text("")
        form.add_date(residence.start_date)
        form.add_date(residence.end_date)
    else:
        form.add_skip(3)

    # previous residence
    previous_cors = CORs(form.applicant.cor).previous
    if previous_cors and len(previous_cors) > 0:
        p1 = previous_cors[0]
        p2 = previous_cors[1] if len(previous_cors) > 1 else None

        form.add_radio(True)
        form.add_dropdown(p1.country, option_lists.country_of_residence)
        form.add_dropdown(p1.status, option_lists.immigration_status)
        form.add_skip(1)
        form.add_date(p1.start_date)
        form.add_date(p1.end_date)

        if p2:
            form.add_dropdown(p2.country, option_lists.country_of_residence)
            form.add_dropdown(p2.status, option_lists.immigration_status)
            form.add_skip(1)
            form.add_date(p2.start_date)
            form.add_date(p2.end_date)
        else:
            form.add_skip(5)
    else:
        form.add_radio(False)
        form.add_skip(10)

    if form_type == "1295":
        if not form.applicant.trcase.same_as_cor:
            form.add_radio(False)
            tr = form.applicant.trcase
            form.add_dropdown(tr.applying_country, option_lists.previous_country)
            form.add_dropdown(tr.applying_status, option_lists.immigration_status)
            if tr.applying_status == "Other":
                form.add_text(tr.other_explain)
            else:
                form.add_skip(1)
            form.add_date(tr.applying_start_date)
            form.add_date(tr.applying_end_date)

        else:
            form.add_radio(True)
            form.add_skip(5)

    marriage = form.applicant.marriage
    status = marriage.marital_status

    form.add_dropdown(status, option_lists.marital_status, True)
    if status.lower() in ["married", "common-law"]:
        form.add_date(marriage.married_date)
        form.add_text(marriage.sp_last_name)
        form.add_text(marriage.sp_first_name)
        if form_type != "1295":
            form.add_radio(marriage.sp_is_canadian)
    else:
        form.add_skip(3 if form_type == "1295" else 4)

    if marriage.previous_married:
        form.add_radio(True)
        form.add_text(marriage.pre_sp_last_name)
        form.add_text(marriage.pre_sp_first_name)
        form.add_dropdown(marriage.pre_relationship_type, option_lists.relation_type)
        form.add_date(marriage.pre_start_date)
        form.add_date(marriage.pre_end_date)
        # speciaially dealing with
        special_YMD_handle(marriage.pre_sp_dob, form)

    else:
        form.add_radio(False)
        form.add_skip(8)


def add_language(form: ApplicationForm):
    """Add language items
    1. native language
    """
    form.add_info("language section")

    form.add_dropdown(
        form.applicant.personal.native_language, option_lists.native_language
    )
    form.add_dropdown(
        form.applicant.personal.english_french, option_lists.communication_language
    )
    if form.applicant.personal.english_french.lower() == "both":
        form.add_text(form.applicant.personal.which_one_better)
    else:
        form.add_skip(1)
    if form.applicant.personal.language_test:
        form.add_radio(True)
    else:
        form.add_radio(False)


def add_passport(form: ApplicationForm):
    """Add passport items
    1. passport number
    2. issue country`
    3. issue date
    4. expiry date
    5. Taiwan
    6. Israeli
    """
    form.add_info("passport section")

    passport: ID = IDs(form.applicant.personid).passport
    form.add_text(passport.number)
    form.add_dropdown(passport.country, option_lists.passport_issue_country, True)
    form.add_date(passport.issue_date)
    form.add_date(passport.expiry_date)

    if "taiwan" in passport.country.lower():
        form.add_radio(True)
    else:
        form.add_skip(1)
    if "israel" in passport.country.lower():
        form.add_radio(True)
    else:
        form.add_skip(1)


def add_national_id(form: ApplicationForm):
    """Add national id items
    1. if has national id
    2. id number
    3. issue country
    4. issue date
    5. expiry date
    """

    form.add_info("national id section")
    national_id: ID = IDs(form.applicant.personid).national_id
    if national_id.number:
        form.add_radio(True)
        form.add_text(national_id.number)
        form.add_dropdown(
            national_id.country, option_lists.passport_issue_country, True
        )
        if national_id.issue_date:
            form.add_date(national_id.issue_date)
        else:
            form.add_skip(1)
        if national_id.expiry_date:
            form.add_date(national_id.expiry_date)
        else:
            form.add_skip(1)
    else:
        form.add_radio(False)
        form.add_skip(4)


def add_uspr_card(form: ApplicationForm):
    form.add_info("US PR card section")

    """Add US PR card items"""
    pr: ID = IDs(form.applicant.personid).pr
    if pr.country and (
        "united states" in pr.country.lower() or "usa" in pr.country.lower()
    ):
        form.add_radio(True)
        form.add_text(pr.number)
        form.add_date(pr.expiry_date)
    else:
        form.add_radio(False)
        form.add_skip(2)


def add_contact_information(form: ApplicationForm, form_type: str = "1295"):
    """Add contact information"""

    form.add_info("contact information section")

    address: Address = Addresses(form.applicant.address).mailing
    residential_address: Address = Addresses(form.applicant.address).residential

    form.add_text(address.po_box)
    form.add_text(address.unit)
    form.add_text(address.street_number)
    form.add_text(address.street_name)
    form.add_text(address.city)
    country = address.country
    form.add_dropdown(country, option_lists.mailing_country)
    if country.lower() == "canada":
        form.add_dropdown(address.province, option_lists.canada_province)
    else:
        form.add_skip(1)
    form.add_text(address.post_code)
    if form_type == "1295":
        form.add_text(address.district)

    if residential_address != address:
        form.add_radio(False)
        form.add_text(residential_address.unit)
        form.add_text(residential_address.street_number)
        form.add_text(residential_address.street_name)
        form.add_text(residential_address.city)
        country = residential_address.country
        form.add_dropdown(country, option_lists.mailing_country)
        if country.lower() == "canada":
            form.add_dropdown(
                residential_address.province, option_lists.canada_province
            )
        else:
            form.add_skip(1)
        form.add_text(residential_address.post_code)
        if form_type == "1295":
            form.add_text(residential_address.district)

    else:
        form.add_radio(True)
        if form_type == "1295":
            form.add_skip(8)
        else:
            form.add_skip(7)

    # telephone
    telephone: Union[Phone, None] = Phones(form.applicant.phone).PreferredPhone
    if telephone and telephone.isCanadaUs:
        pause = 0.5  # this place could be easily erred
        form.add_checkbox(True)
        form.add_skip(1)
        form.add_dropdown(telephone.variable_type.title(), option_lists.telephone_type)
        form.add_skip(1)
        if form_type == "1295":  # try to work around 1295 tab order issue
            form.add_text(telephone.ext, pause=pause)
            form.add_text(telephone.number, pause=pause)
            form.add_skip(3)
        else:
            # when a valid Canada phone number finished, it will automatically tab to next control
            ext = telephone.ext if telephone.ext else ""
            form.add_text(telephone.number + ext, pause=pause)

    else:
        pause = 0.1
        form.add_skip(1)
        form.add_checkbox(True)
        form.add_dropdown(telephone.variable_type.title(), option_lists.telephone_type)
        form.add_text(telephone.country_code, pause=pause)
        form.add_text(telephone.number, pause=pause)
        form.add_text(telephone.ext, pause=pause)
    # alternative phone or fax is not used usually, ignore alternate telephone and fax
    form.add_skip(11)

    form.add_text(form.applicant.personal.email)


def add_coming_into_canada(form: ApplicationForm):
    """Add coming into canada."""

    form.add_info("coming into Canada section")

    coming = form.applicant.trcasein
    form.add_date(coming.original_entry_date)
    form.add_text(coming.original_entry_place)
    form.add_dropdown(coming.original_purpose, option_lists.purpose_of_coming)

    if coming.original_purpose == "Other":
        form.add_text(coming.original_other_reason)
    else:
        form.add_skip(1)

    if coming.most_recent_entry_date:
        form.add_date(coming.most_recent_entry_date)
        form.add_text(coming.most_recent_entry_place)
        form.add_text(coming.doc_number)
    else:
        form.add_skip(3)


def add_work_detail(form: ApplicationForm, form_type: str = "1295"):
    """Add details of intended work in Canada"""

    form.add_info("intended work section")

    work_detail = (
        form.applicant.wp if form_type == "1295" else form.applicant.wpincanada
    )
    if form_type == "5710":
        form.add_dropdown(work_detail.work_permit_type, option_lists.work_permit_ex)
        if work_detail.work_permit_type == "Other":
            form.add_text(work_detail.other_explain)
        else:
            form.add_skip(1)
    else:
        form.add_dropdown(work_detail.work_permit_type, option_lists.work_permit)

    if work_detail.employer_name:
        form.add_text(work_detail.employer_name)
    else:
        form.add_text("Not applicable")

    if work_detail.employer_name:
        form.add_text(work_detail.employer_address)
    else:
        form.add_text("Not applicable")

    province = work_detail.work_province
    if province:
        form.add_dropdown(province, option_lists.canada_province)
    else:
        form.add_skip(1)

    if work_detail.work_city:
        form.add_dropdown(
            work_detail.work_city.upper(), option_lists.canada_province_cities[province]
        )
    else:
        form.add_skip(1)

    if work_detail.employer_address:
        form.add_text(work_detail.employer_address)
    else:
        form.add_text("TO BE DETERMINED")

    if work_detail.job_title:
        form.add_text(work_detail.job_title)
    else:
        form.add_text("TO BE DETERMINED")

    if work_detail.brief_duties:
        form.add_text(work_detail.brief_duties)
    else:
        form.add_text("TO BE DETERMINED")

    if work_detail.duration_from:
        form.add_date(work_detail.duration_from)
    else:
        form.add_skip(1)

    if work_detail.duration_to:
        form.add_date(work_detail.duration_to)
    else:
        form.add_skip(1)

    if work_detail.lmia_num_or_offer_num:
        form.add_text(work_detail.lmia_num_or_offer_num)
    else:
        form.add_skip(1)

    if form_type == "5710":
        if work_detail.caq_number:
            form.add_text(work_detail.caq_number)
            form.add_date(work_detail.expiry_date)
        else:
            form.add_skip(2)
        form.add_radio(work_detail.pnp_certificated)


def add_visit_detail(form: ApplicationForm):
    """Add visit details"""
    form.add_info("details of visit section")

    visit = form.applicant.vrincanada
    form.add_dropdown(visit.visit_purpose, option_lists.purpose_of_vist)
    if visit.visit_purpose.lower() == "other":
        form.add_text(visit.other_explain)
    else:
        form.add_skip(1)
    form.add_date(visit.duration_from)
    form.add_date(visit.duration_to)
    form.add_text(str(visit.funds_available))
    form.add_dropdown(visit.paid_by, option_lists.expense_by)
    if visit.paid_by.lower() == "other":
        form.add_text(visit.other_payer_explain)
    else:
        form.add_skip(1)

    form.add_text(visit.name1)
    form.add_text(visit.relationship1)
    form.add_text(visit.address1)
    form.add_text(visit.name2)
    form.add_text(visit.relationship2)
    form.add_text(visit.address2)


def add_study_detail(form: ApplicationForm):
    """Add study details"""
    form.add_info("details of study section")

    detail = form.applicant.spincanada
    form.add_text(detail.school_name)
    form.add_dropdown(
        option_lists.get_5709_study_level(detail.study_level),
        option_lists.level_of_study,
    )
    form.add_dropdown(detail.study_field, option_lists.field_of_study, True)

    form.add_dropdown(detail.province, option_lists.canada_province)
    form.add_dropdown(
        detail.city.upper(), option_lists.canada_province_cities[detail.province]
    )
    form.add_text(detail.address)
    form.add_text(detail.dli)
    form.add_text(detail.student_id)
    form.add_date(detail.duration_from)
    form.add_date(detail.duration_to)

    form.add_text(detail.tuition_cost)
    form.add_text(detail.room_cost)
    form.add_text(detail.other_cost)
    form.add_text(detail.fund_available)
    form.add_dropdown(detail.paid_person, option_lists.expense_by)
    form.add_skip(1)
    form.add_radio(detail.apply_work_permit)
    if detail.apply_work_permit:
        form.add_dropdown(detail.work_permit_type, option_lists.work_permit_student)
        form.add_text(detail.caq_number)
        form.add_text(detail.expiry_date)
    else:
        form.add_skip(3)


def add_education(form: ApplicationForm):
    """Add education"""
    form.add_info("education section")

    education: Union[EducationBase, None] = EducationHistory(
        form.applicant.education
    ).highestEducation
    if education and education.level_in_num >= 2:  # >=2 means post secondary
        form.add_radio(True)
        form.add_date(education.start_date, True)
        form.add_text(education.field_of_study)
        form.add_text(education.school_name)
        form.add_date(education.end_date, True)
        form.add_text(education.city)
        country = education.country
        form.add_dropdown(country, option_lists.country_of_birth)
        if country and country.lower() == "canada":
            form.add_dropdown(education.province, option_lists.canada_province)
        else:
            form.add_skip(1)
    else:
        form.add_radio(False)
        form.add_skip(9)


def add_employment(form: ApplicationForm, form_type: str = "1295"):
    """Add employment"""
    form.add_info("employment section")

    slots = 3  # num of slots in form
    if form.applicant.employment:
        employments = form.applicant.employment
        for employment in employments:
            # WTF!! work around the form bug
            if slots == 2 and form_type == "1295":
                year, month, day = employment.start_date.strftime("%Y-%m-%d").split("-")
                form.add_text(year)
                form.add_text(month)
            else:
                form.add_date(employment.start_date, True)
            form.add_text(employment.job_title)
            form.add_text(employment.company)
            form.add_date(employment.end_date, True)
            form.add_text(employment.city)
            country = employment.country
            form.add_dropdown(country, option_lists.country_of_birth)
            if country.lower() == "canada":
                form.add_dropdown(employment.province, option_lists.canada_province)
            else:
                form.add_skip(1)
            slots -= 1
            if slots == 0:
                break

    # skip emply slots
    for _ in range(slots):
        form.add_skip(9)


def add_background(form: ApplicationForm):
    """Add background information"""
    form.add_info("background information section")

    background: TrBackground = form.applicant.trbackground
    form.add_radio(background.q1a)
    form.add_radio(background.q1b)
    if background.q1a or background.q1b:
        form.add_text(background.q1c)
    else:
        form.add_skip(1)

    form.add_radio(background.q2a)
    form.add_radio(background.q2b)
    form.add_radio(background.q2c)
    if background.q2a or background.q2b or background.q2c:
        form.add_text(background.q2d)
    else:
        form.add_skip(1)

    form.add_radio(background.q3a)
    if background.q3a:
        form.add_text(background.q3b)
    else:
        form.add_skip(1)

    form.add_radio(background.q4a)
    if background.q4a:
        form.add_text(background.q4b)
    else:
        form.add_skip(1)

    form.add_radio(background.q5)

    form.add_radio(background.q6)


def add_signature(form: ApplicationForm):
    """Add signature section"""
    form.add_radio(True)

    form.add_text(
        f"{form.applicant.personal.last_name} {form.applicant.personal.first_name}"
    )  # signature
    form.add_text(str(date.today()))


def build_form_1295(applicant: dict) -> ApplicationForm:
    """Build form 1295 with Applicant data

    Args:
        applicant: applicant data dict

    Returns:
        ApplicationForm of 1295
    """
    print("Build form 1295")

    form = ApplicationForm(applicant)

    # skip the top buttons
    form.add_skip(3)

    # Header section
    add_header(form)

    # Personal Details section
    add_personal_detail(form, "1295")

    # Language section
    add_language(form)

    # Passport section
    add_passport(form)

    # National Identity document section
    add_national_id(form)

    # US PR Card section
    add_uspr_card(form)

    # Contact Information section
    add_contact_information(form, "1295")

    # Details of intended work in Canada section #TODO:
    add_work_detail(form)

    # Education section
    add_education(form)

    # Employment section
    add_employment(form)

    form.add_info("skip clear section button")
    form.add_skip(1)  # skip Clear Section button

    # Background information section
    add_background(form)

    # Signature section
    add_signature(form)

    form.add_skip(1)  # skip infosource web link
    form.add_info("click validate button")
    form.add_button()

    return form


def build_form_5708(applicant: dict) -> ApplicationForm:
    form = ApplicationForm(applicant)

    # skip the top buttons
    form.add_skip(3)

    add_header(form, "5708")

    add_personal_detail(form)

    add_language(form)

    add_passport(form)

    add_national_id(form)

    add_uspr_card(form)

    add_contact_information(form, "5708")

    add_coming_into_canada(form)

    add_visit_detail(form)

    add_education(form)

    add_employment(form, "5708")

    form.add_info("skip clear section button")
    form.add_skip(1)  # skip Clear Section button

    # Background information section
    add_background(form)

    # Signature section
    add_signature(form)

    form.add_skip(1)  # skip infosource web link
    form.add_info("click validate button")
    form.add_button()

    return form


def build_form_5709(applicant: dict) -> ApplicationForm:
    form = ApplicationForm(applicant)

    # skip the top buttons
    form.add_skip(3)

    add_header(form, "5709")

    add_personal_detail(form)

    add_language(form)

    add_passport(form)

    add_national_id(form)

    add_uspr_card(form)

    add_contact_information(form, "5709")

    add_coming_into_canada(form)

    add_study_detail(form)

    add_education(form)

    add_employment(form, "5709")

    form.add_info("skip clear section button")
    form.add_skip(1)  # skip Clear Section button

    add_background(form)

    add_signature(form)

    form.add_skip(1)  # skip infosource web link
    form.add_info("click validate button")
    form.add_button()

    return form


def build_form_5710(applicant: dict) -> ApplicationForm:
    form = ApplicationForm(applicant)

    # skip the top buttons
    form.add_skip(3)

    add_header(form, "5710")

    add_personal_detail(form)

    add_language(form)

    add_passport(form)

    add_national_id(form)

    add_uspr_card(form)

    add_contact_information(form, "5710")

    add_coming_into_canada(form)

    add_work_detail(form, "5710")

    add_education(form)

    add_employment(form, "5710")

    form.add_info("skip clear section button")
    form.add_skip(1)  # skip Clear Section button

    add_background(form)

    add_signature(form)

    form.add_skip(1)  # skip infosource web link
    form.add_info("click validate button")
    form.add_button()

    return form


form_builder = {
    "1295": build_form_1295,
    "5708": build_form_5708,
    "5709": build_form_5709,
    "5710": build_form_5710,
}


def build_form(form_type: str, applicant: dict) -> ApplicationForm:
    return form_builder[form_type](applicant)



from docx import Document
from docx.enum.text import WD_PARAGRAPH_ALIGNMENT
from docx.enum.section import WD_SECTION_START,WD_ORIENT
from docx.enum.style import WD_STYLE_TYPE
from docx.shared import Inches, Cm, Pt, RGBColor



contents=[
    {
        "block":"section",
        "style":{}
    },
    {
        "block":"styles",
        "style":{}
    },
    {
        "block":"heading",
        "level":1,
        "text":"Submission Letter",
        "style":{}
    },
    {   
        "block":"paragraph",
        "text":"Underline is a bit of a special case. It is a hybrid of a tri-state property and an enumerated value property. True means single underline, by far the most common. False means no underline, but more often {} is the right choice if no underlining is wanted since it is rare to",
        "style":{}
    },
    {
        "block":"heading",
        "level":2,
        "text":"ParagraphFormat",
        "style":{}
    },
    {
        "block":"paragraph",
        "text":"Here’s an example of how you would create a paragraph style having hanging indentation of 1/4 inch, 12 points spacing above, and widow/orphan control:",
        "style":{
            "first_line_indent":{
                "unit":"inch",
                "value":0.25
            }
        }
    },
    {
        "block":"heading",
        "level":3,
        "text":"ParagraphFormat",
        "style":{}
    },
    {
        "block":"paragraph",
        "text":"Here’s an example of how you would create a paragraph style having hanging indentation of 1/4 inch, 12 points spacing above, and widow/orphan control:",
        "style":{
            "left_indent":{
                "unit":"inch",
                "value":0.5
            }
        }
    },
    {
        "block":"run",
        "text":"Jacky",  
        "style":{"name":"Arial"}
    },
    {
        "block":"table",
        "text":[
            ['name','sex','age','education'],
            ['jacky','Male','52','Master'],
            ['dan','male','11','elementory']
        ],  
        "style":"Medium Shading 1 Accent 2"
    }
]   

class Word():
    default_styles={
        "Normal":{
            "name":"Calibri",
            "size":Pt(11),
            "color":RGBColor(0,0,0)
        },
        "Heading 1":{
            "name":"Arial Black",
            "size":Pt(36),
            "color":RGBColor(0,0,125)
        },
        "Heading 2":{
            "name":"Arial Black",
            "size":Pt(24),
            "color":RGBColor(0,125,0)
        },
        "Heading 3":{
            "name":"Arial Black",
            "size":Pt(18),
            "color":RGBColor(0,125,0)
        }
        
    }
    default_paragraph_format={
        "name":"Indent",
        "left_indent":{
            "unit":"inch",
            "value":0
            },
        "first_line_indent":{
            "unit":"inch",
            "value":0
        },
        "space_before":{
            "unit":"pt",
            "value":12
        },
        "space_after":{
            "unit":"pt",
            "value":18
        },
        "widow_control":True
    }
    default_run_format={
        "name":'Calibri',
        "size":Pt(36),
        "italic":True,
        "bold":True,
        "underline":True,
        "color":RGBColor(0,0,255)
    }

    def __init__(self,contents,fileName):
        self.content=contents
        self.fileName=fileName
        self.document=Document()

    def __getUnitValue(self,format,key):
        if  format[key].get("unit","pt")=='pt':
            return Pt(format[key].get("value",None))
        elif format[key].get("unit","pt")=='inch':
            return Inches(format[key].get("value",None))
        elif format[key].get("unit","pt")=='cm':
            return Cm(format[key].get("value",None))
        else:
            return None
    # 2D list
    def __creatTable(self,dataList,style):
        rows=len(dataList)
        cols=len(dataList[0])
        table=self.document.add_table(rows=rows,cols=cols)
        table.style=style
        for row in range(0,rows):
            for col in range(0,cols):
                cell=table.cell(row,col)
                cell.text=dataList[row][col]

    def create(self):
        lastParagraph=None
        for content in contents:
            if content['block']=='styles':
                # if content has its own format, overwrite the counterpart default format
                styleData={**Word.default_styles,**content.get('styles',{})}
                print("document style: ", styleData)
                for k, v in Word.default_styles.items():
                    font=self.document.styles[k].font
                    font.name=v.get('name',None)
                    font.size=v.get('size',None)
                    font.color.rgb=v.get('color',None)
            elif content['block']=='paragraph':
                format={**Word.default_paragraph_format,**content.get('style',{})}
                print('paragraphp format: ',format)
                text=content.get('text',"")
                paragrah=self.document.add_paragraph(text)
                # paragraph formatting
                paragraph_format=paragrah.paragraph_format
                paragraph_format.space_before = self.__getUnitValue(format,"space_before")
                paragraph_format.space_after=self.__getUnitValue(format,"space_after")
                paragraph_format.left_indent=self.__getUnitValue(format,"left_indent")
                paragraph_format.first_line_indent=self.__getUnitValue(format,"first_line_indent")
                paragraph_format.widow_control =format.get("widow_control",True)
                lastParagraph=paragrah

            elif content['block']=='heading':
                self.document.add_heading(content.get('text',''),level=content.get("level",None))
            elif content['block']=='table':
                self.__creatTable(content.get("text",[]),content.get("style","Normal"))
            elif content['block']=='run':
                run=lastParagraph.add_run(content.get("text",''))
                format={**Word.default_run_format,**content.get('style',{})}
                print("run format: ",format)
                font=run.font
                font.name=format.get("name","Calibri")
                font.size=format.get("size",None)
                font.italic=format.get("italic",None)
                font.bold=format.get("bold",None)
                font.underline=format.get("underline",None)
                font.color.rgb=format.get("color",RGBColor(0,0,255))
            else:
                pass 
        self.document.save(self.fileName)

w=Word(contents,"jacky.docx")
w.create()



# import context
import argparse,os
from termcolor import colored
from model.lmia.submissionletter import SubmissionLetterModel
from model.lmia.employertraining import EmployerTrainingModel
from model.recruit.recruitmentsummary import RecruitmnetSummaryModel,RecruitmnetSummaryDocxAdaptor
    
def main():
    """ arguments:
        -e: excel file as source
        -t: output word file name
        -d: --> sl: submission letter, et: employer training guideline, rs: recruitment summary 
    """
    parser=argparse.ArgumentParser(description="used for processing lmia application")
    # parser.add_argument("-s", "--stream", help="input stream code. 5593 for pr, 5626 for HWS or 5627 for LWS ")
    parser.add_argument("-e", "--excel", help="input excel file name including the data for your specific stream")
    parser.add_argument("-t", "--to", help="input docx file name for output")
    parser.add_argument("-d", "--document", help="input which kind of document to generate. type list: sl: submission letter et: employer training guideline, rs: recruitment summary ")
    args = parser.parse_args()

    if args.excel and args.to:
        match args.document: 
            case 'sl':
                # if input excel file, then generate it
                if not os.path.isfile(args.excel):
                    SubmissionLetterModel(output_excel_file=args.excel)
                    print(colored(f'{args.excel} is not existed, so we created the excel file based on your data structure.','green'))
                    print(colored('Please fill the excel with data and do it again','yellow'))
                    return 
                context=SubmissionLetterModel(excels=[args.excel])
                context.makeDocx(args.to)
    
            case 'et':
                # if input excel file, then generate it
                if not os.path.isfile(args.excel):
                    EmployerTrainingModel(output_excel_file=args.excel)
                    print(colored(f'{args.excel} is not existed, so we created the excel file based on your data structure.','green'))
                    print(colored('Please fill the excel with data and do it again','yellow'))
                    return 
                context=EmployerTrainingModel(excels=[args.excel])
                context.makeDocx(args.to)
        
            case 'rs':
                # if input excel file, then generate it
                if not os.path.isfile(args.excel):
                    RecruitmnetSummaryModel(output_excel_file=args.excel)
                    print(colored(f'{args.excel} is not existed, so we created the excel file based on your data structure.','green'))
                    print(colored('Please fill the excel with data and do it again','yellow'))
                    return 
                context=RecruitmnetSummaryModel(excels=[args.excel])
                RecruitmnetSummaryDocxAdaptor(context).make(args.to)
    
if __name__=='__main__':
    main()






from .context import BASEDIR
from model.common.commonmodel import CommonModel
from pydantic import BaseModel, EmailStr
from datetime import date
from typing import Optional, Union, List
import os


class Facts(BaseModel):
    dob: date
    # asset
    liquid_assets: Optional[float]
    net_worth: Optional[float]
    
    #language
    reading: Optional[Union[int, float]]
    writting: Optional[Union[int, float]]
    speaking: Optional[Union[int, float]]
    listening: Optional[Union[int, float]]
    test_format: Optional[str]
    
    #work experience
    start_date: date
    end_date: date
    work_noc: str
    term: str = "Fulltime"
    work_country: str = "Other"
    job_title: str
    work_province: str = "Other"
    share_percentage: float = 0
    
    # edu
    level: str = "Bachelor"
    edu_country: str = "Other"
    edu_province: str = "Other"
    graduate_date: date

    # relative
    relative_dob: Optional[date]
    relative_province: Optional[str]
    canada_status: Optional[str]
    relationship: Optional[str]
    
    # job offer
    job_offer:bool
    joboffer_noc:Optional[str]
    joboffer_province:Optional[str]


class QuickAssessModel(CommonModel):
    facts: Facts

    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self, excels=None, output_excel_file=None):
        if output_excel_file:
            excels=self.getExcels(["excel/quickassess.xlsx"])
        else:
            if excels is None and len(excels) == 0:
                raise ValueError(
                    'You must input excel file list as source data for validation')
        # call parent class for validating
        super().__init__(excels, output_excel_file, globals())



from pydantic import BaseModel
from typing import Optional, List
from collections import namedtuple
from model.common.mixins import EqMixin


class Phone(BaseModel, EqMixin):
    variable_type: str
    display_type: str
    country_code: Optional[str]
    number: Optional[str]
    ext: Optional[str]

    class Config:
        anystr_lower = True

    @property
    def international_format_full(self):
        return "+" + self.country_code + " " + self.number

    @property
    def isCanadaUs(self):
        return self.country_code and self.country_code == "1"

    @property
    def is_fax(self):
        return self.v_type == "fax"

    @property
    def NA_format_number(self):
        if len(self.number) != 10:
            raise ValueError(f"{self.number} is not a valid North America phone number")
        return "(" + self.number[:3] + ") " + self.number[3:6] + "-" + self.number[6:]

    # Return a list including area code, first part and second part of number
    @property
    def NA_format_list(self):
        if len(self.number) != 10:
            raise ValueError(f"{self.number} is not a valid North America phone number")
        return [self.number[:3], self.number[3:6], self.number[6:]]

    # Return a list including area code, first part and second part of number
    @property
    def NA_format_namedtuple(self):
        if len(str(self.number)) != 10:
            raise ValueError(f"{self.number} is not a valid North America phone number")
        NA_Number = namedtuple("NA_Number", ["area_code", "part1", "part2"])
        number = NA_Number(self.number[:3], self.number[3:6], self.number[6:])
        return number

    def __str__(self):
        return self.international_format_full


class Phones(object):
    def __init__(self, phone_list: List[Phone]) -> None:
        self.phonees = phone_list

    def _specific_phone(self, v_type):
        phone = [phone for phone in self.phonees if phone.variable_type == v_type]
        return phone[0] if phone else None

    @property
    def cellular(self):
        return self._specific_phone("cellular")

    @property
    def residential(self):
        return self._specific_phone("residential")

    @property
    def business(self):
        return self._specific_phone("business")

    @property
    def working(self):
        return self._specific_phone("working")

    @property
    def PreferredPhone(self):
        for Phone_type in ["cellular", "residential", "business", "working"]:
            phone = self._specific_phone(Phone_type)
            if phone:
                return phone

    def getPreferredPhoneFromList(self, type_list):
        for Phone_type in type_list:
            phone = self._specific_phone(Phone_type)
            if phone:
                return phone



#!/Users/jacky/tools/venv/bin/python3.9

#!/Users/jacky/tools/venv/bin/python3.9
# This app is developed for daily files manupilation 
from glob import glob
import argparse,os,shutil,csv
from datetime import datetime


def collectNames(input):
    # get sorted file list by name
    names=sorted(glob(input))
    # create a tmp text file
    now=datetime.now()
    tmpFile=str(now.year)+str(now.month)+str(now.day)+str(now.hour)+str(now.minute)+str(now.second)+".txt"
    with open(tmpFile,"w") as f:
        for name in names:
            f.write(f"{name}\n")
    
    os.system(f'code {tmpFile}')
    return names,tmpFile

def batchChange(oldNames,txtFile):
    with open(txtFile,"r") as f:
        newNames=f.readlines()
    match=dict(zip(oldNames,newNames))
    for oldName,newName in match.items():
        newName=newName.replace("\n","")
        os.system(f"mv '{oldName}' '{newName}'")
        print(f'Successfully changed {oldName} to {newName}...')
    os.system(f"rm {txtFile}")
    print('Done!')



def main():
    parser=argparse.ArgumentParser(description="For munipulate pdfs: split, merge, reverse, rotate,img double way process...")
    
    parser.add_argument("-bc", "--batchchange", help="get matching files by using filter, and then change their filenames in a text editor (exp: '*.jpg'. '' is must!) ")


    args = parser.parse_args()

    if args.batchchange:
        names,txtFN=collectNames(args.batchchange)
        change=input('Are you ready to change?')
        if change[0].upper()=="Y":
            batchChange(names,txtFN)
            return
        else:
            os.system(f"rm {txtFN}")
            return 
    
if __name__=="__main__":
    main()



from datetime import date
from pydantic import BaseModel, EmailStr, validator
from typing import Optional
from model.common.educationbase import EducationBase
from model.common.person import Person
from model.common.utils import normalize
from pydantic.class_validators import root_validator


class Personal(Person):
    used_last_name: Optional[str]
    used_first_name: Optional[str]
    uci: Optional[str]
    country_of_birth: str
    place_of_birth: str
    email: EmailStr

    native_language: str
    english_french: str
    which_one_better: Optional[str]
    language_test: bool

    _normalize_used_first_name = validator(
        "used_first_name", allow_reuse=True, check_fields=False
    )(normalize)
    _normalize_used_last_name = validator(
        "used_last_name", allow_reuse=True, check_fields=False
    )(normalize)

    # @root_validator
    # def checkAnswers(cls,values):
    #     questions=['english_french']
    #     explanations=['which_one_better']
    #     qas=dict(zip(questions,explanations))
    #     for k,v in qas.items():
    #         if values.get(k) and not values.get(v):
    #                 raise ValueError(f"Since {k} is true, but you did not answer the question {v} in info-position sheet")
    #     return values


class Marriage(BaseModel):
    marital_status: str
    married_date: Optional[date]
    sp_last_name: Optional[str]
    sp_first_name: Optional[str]
    sp_is_canadian: Optional[bool]
    previous_married: bool
    pre_sp_last_name: Optional[str]
    pre_sp_first_name: Optional[str]
    pre_relationship_type: Optional[str]
    pre_sp_dob: Optional[date]
    pre_start_date: Optional[date]
    pre_end_date: Optional[date]


class PersonId(BaseModel):
    variable_type: str
    display_type: str
    number: Optional[str]
    country: Optional[str]
    issue_date: Optional[date]
    expiry_date: Optional[date]


# Not everyone has education. So, it's optional. Without start date and end date, the app will regard having no education
class Education(EducationBase):
    city: Optional[str]
    province: Optional[str]
    country: Optional[str]


# Not everyone has employment experience. So, it's optional. Without start date and end date, the app will regard having no post secondary education
class Employment(BaseModel):
    start_date: Optional[date]
    end_date: Optional[date]
    job_title: Optional[str]
    company: Optional[str]
    city: Optional[str]
    province: Optional[str]
    country: Optional[str]


class Travel(BaseModel):
    start_date: Optional[date]
    end_date: Optional[date]
    length: Optional[int]
    destination: Optional[str]
    purpose: Optional[str]


class Family(BaseModel):
    last_name: str
    first_name: str
    native_last_name: str
    native_first_name: str
    marital_status: str
    date_of_birth: Optional[date]
    # place_of_birth:str
    birth_country: str
    # country_of_citizenship:str
    address: str
    occupation: str
    relationship: str
    # email:Optional[EmailStr]
    # date_of_death:Optional[date]
    # place_of_death:Optional[str]
    accompany_to_canada: bool


# Countries of Residence
class COR(BaseModel):
    start_date: date
    end_date: Optional[date]
    country: str
    status: str



from functools import reduce
from pydantic import validator
from datetime import date
from model.common.utils import Duration

# Compare if self obj and another obj is same with compare all property and its value
class EqMixin(object):
    def __eq__(self,another):
        for k,v in self.__dict__.items():
            if v != getattr(another,k,None):
                return False
        return True

class KwargsMixin(object):
    def _getInput(self,global_dict,excel_obj):
        """
        Prerequisite of this mixin
        1. excel_obj is the one that all input excels combined, no sheet has same name
        2. Final level model is constructed with all sheets/tables, no specific single varialbe
        3. Only InfoSheet and TableList two data structure in the final level model
        """
        input_kwargs={}
        # According to the prerequisite described above, all the referrenced scheme are defined in 'definitions'. 
        # key is Model name, and after lower(), it is also excel sheet name and the property defined in this class. They are same. This is the requirement.
        for k in self.schema()['definitions'].keys(): 
            # globals() returns a dict inlcuding all global classes, methods, ....
            klass=global_dict.get(k)  
            # use k.lower (it's a sheet name here) to get sheet data
            sheet_data=excel_obj.dict.get(k.lower(),None)   
            # only two types, one is list of model objects,another is model object
            if not isinstance(sheet_data,list):
                # assign the input kwargs. Here k.lower() is class's property. klass is the model, on which the app create an obj
                input_kwargs[k.lower()]=klass(**sheet_data)
            else:
                input_kwargs[k.lower()]=[klass(**d) for d in sheet_data]
        return input_kwargs

class MakeExcelMixin(object):
    def _generateExcel(self,global_dict,excel_obj,excel_file):    
        sheets=[]
        tables=[]
        # 获得每一sheet对应class，并获得它的properties list
        for k in self.schema()['definitions'].keys(): 
            # globals() returns a dict inlcuding all global classes, methods, ....
            klass=global_dict.get(k) #get the class
            variables=list(klass.__fields__.keys()) # get all variables in the shhet or table
            sheet_table_name=k.lower() # k.lower is the sheet / table name according to convention
            if 'info-'+sheet_table_name in excel_obj.sheet_names:
                sheets.append(excel_obj.getSheet(sheet_table_name,variables)) # get sheet object 
            if 'table-'+sheet_table_name in excel_obj.table_names:
                tables.append(excel_obj.getTable(sheet_table_name,variables)) # get table object
                
        # 根据最终更新了variables的sheets和talbes，生成新的excel文件
        excel_obj.makeExcel(excel_file,sheets=sheets,tables=tables)

# 处理所包含start_date 和 end_date的记录。 
class DurationMixin(object):
    @property
    def start_from(self):
        return self.start_date.strftime('%b %Y')

    @property
    def end_to(self):
        return self.end_date.strftime('%b %Y') if self.end_date else "Present"
    
    @property
    def lengthOfYears(self):
        return Duration(self.start_date,self.end_date).years
    


from model.tr.d5257 import data_model, convert_model,remove_model
from model.tr.pdfformmodel import PdfFormModel


class F5257Model(PdfFormModel):
    
    def __init__(self,xml_file):
        # pass the globals variables to base class
        model={'data_model':data_model,'convert_model':convert_model,'remove_model':remove_model}
        super().__init__(xml_file,model)
            
    
    def _getSpecial(self):
        # work 
        occupation_info=[e for e in self.tree.iterfind('Page3/Occupation')][0]
        occupations=[]
        for i in [1,2,3]:
            occupation_item=occupation_info.find('OccupationRow'+str(i))
            occupation=self._getSubItem(occupation_item,data_model['occupation'])
            occupations.append(occupation)

        self.data['occupations']=occupations

        
#     def _convert(self):
#         # 1 convert countries
#         country_pairs={
#             "trcase":["applying_country"],
#             "personal":['country_of_birth','citizen'],
#             'passport':['country'],
#             'national_id':['country'],
#             'education':['country'],
#             'cor':['current_cor_country','previous_cor_country1','previous_cor_country2']
#         }
#         language_pairs={
#             'personal':['native_language']
#         }
#         other_pairs={
#             'phone':["variable_type"],
#             'altphone':["variable_type"]
#         }
#         canada_city_pairs={
#             # 'sp':["city"]
#         }
#         tr_canada_status_pairs= {
#             'cor':['current_cor_status','previous_cor_status1','previous_cor_status2']
#         }
#         tr_marital_status_pairs={
#             'marriage':['marital_status']
#         }
        
#         tr_pre_marital_status_pairs={
#             'marriage':['pre_relationship_type']
#         }
        
#         self._execConvert(country_pairs,country)
#         self._execConvert(language_pairs,language)
#         self._execConvert(other_pairs,phone_type)
#         self._execConvert(canada_city_pairs,canada_city)
#         self._execConvert(tr_canada_status_pairs,tr_canada_status)
#         self._execConvert(tr_marital_status_pairs,tr_marital_status)
#         self._execConvert(tr_pre_marital_status_pairs,tr_marital_status)
    
#     def _assembl(self):
#         # for English or French
#         self.data['personal']['which_one_better']='English' if self.data['personal']['which_one_better']=='01' else "French"
#         # assembly dob 
#         self.data['personal']['dob']=self.data['personal'].get('dob_year')+"-"+self.data['personal'].get('dob_month')+"-"+self.data['personal'].get('dob_day')
#         dob_y,dob_m,dob_d=self.data['marriage'].get('pre_sp_dob_year'),self.data['marriage'].get('pre_sp_dob_month'),self.data['marriage'].get('pre_sp_dob_day')
#         self.data['marriage']['pre_sp_dob']=dob_y+"-"+dob_m+"-"+dob_d  if all([dob_y,dob_m,dob_d]) else None
        
#         self._cor_assemble()
#         self._id_assemble()
#         self._address_assemble()
#         self._phone_assemble()
#         # self._work_permit_assemble()
#         self._education_assemble()
#         self._occupation_assemble()
        
#     def _remove(self):
#         popup_pairs={
#             'personal':['dob_year','dob_month','dob_day'],
#             'marriage':['pre_sp_dob_year','pre_sp_dob_month','pre_sp_dob_day']
#         }
#         for k,v in popup_pairs.items():
#             [self.data[k].pop(item) for item in v]
            
#     def show(self,sheet):
#         # show off
#         if isinstance(sheet,dict):
#             for variable, text in sheet.items():
#                 print(variable,text)
#         elif isinstance(sheet,list):
#             for i in range(len(sheet)):
#                 for variable, text in sheet[i].items():
#                     print(variable,text)
                    
#     # countries of residence
#     def _cor_assemble(self):
#                 # assemble countries of residence
#         cor_dict=self.data['cor']
#         cor_current={
#             'country':cor_dict['current_cor_country'],
#             'status':cor_dict['current_cor_status'],
#             'other':cor_dict['current_cor_other'],
#             'start_date':cor_dict['current_cor_start_date'],
#             'end_date':cor_dict['current_cor_end_date']
#             }
#         cor_pre1={
#             'country':cor_dict['previous_cor_country1'],
#             'status':cor_dict['previous_cor_status1'],
#             'other':cor_dict['previous_cor_other1'],
#             'start_date':cor_dict['previous_cor_start_date1'],
#             'end_date':cor_dict['previous_cor_end_date1']
#             }
#         cor_pre2={
#             'country':cor_dict['previous_cor_country2'],
#             'status':cor_dict['previous_cor_status2'],
#             'other':cor_dict['previous_cor_other2'],
#             'start_date':cor_dict['previous_cor_start_date2'],
#             'end_date':cor_dict['previous_cor_end_date2']
#             }
#         # re-sort the order of cor 
#         cor=[]
#         for c in [cor_current,cor_pre1,cor_pre2]:
#             d={}
#             for key in ['start_date','end_date','country','status']:
#                 d[key]=c[key]
#             cor.append(d)
#         self.data['cor']=cor

#     # ids 
#     def _id_assemble(self):
#         passport={'variable_type':'passport',**self.data['passport']}
#         nid={'variable_type':'id',**self.data['national_id']}
#         us_pr={
#             'variable_type':'pr',
#             'number':self.data['us_pr']['number'],
#             'expiry_date':self.data['us_pr']['expiry_date']
#             }
#         self.data['personid']=[passport,nid,us_pr]
#         [self.data.pop(x) for x in ['passport','national_id','us_pr']]

#     # address 
#     def _address_assemble(self):
#         m_dict=self.data['mailing_address']
#         r_dict=self.data['residential_address']
#         m={'variable_type':'mailing_address',**m_dict}
#         if r_dict['same_as_mailing']=='Y':
#             r=m
#         else:
#             r={'variable_type':"residential_address",**r_dict}
#         self.data['address']=[m,r]
#         [self.data.pop(x) for x in ['mailing_address','residential_address']]
    
#      # phone 
#     def _phone_assemble(self):
#         temp_phone=[]
#         for p_type in ['phone','altphone','fax']:
#             d=self.data[p_type]
#             if d.get('number'): #如果没有号码，就位空
#                 d['variable_type']='fax' if not d.get('variable_type') else d['variable_type']
#                 temp_phone.append({ k:v for k,v in d.items() if k in ['variable_type','country_code','number','ext']})
#         [self.data.pop(x) for x in ['phone','altphone','fax']]
#         self.data['phone']=temp_phone        

#      # work permit 
#     def _work_permit_assemble(self):
#         temp_wp={}
#         for d in [self.data['detailed_work'],self.data['intended_location'],self.data['intended_job']]:
#             temp_wp={**temp_wp,**{ k:v for k,v in d.items()}}
#         [self.data.pop(x) for x in ['detailed_work','intended_location','intended_job']]
#         self.data['wp']=temp_wp
        
#     # education
#     def _education_assemble(self):
#         self.data['education']['start_date']=self.data['education']['start_date_year']+'-'+self.data['education']['start_date_month']+'-01'
#         self.data['education']['end_date']=self.data['education']['end_date_year']+'-'+self.data['education']['end_date_month']+'-01'
#         # re-sort education's order
#         temp={}
#         for k in ['start_date','end_date','school_name','education_level','field_of_study','city','province','country']:
#             temp[k]=self.data.get('education').get(k,None)
#         self.data['education']=temp
#     # occupation
#     def _occupation_assemble(self):
#         temp=[]
#         for occupation in self.data['occupations']:
#             occupation['start_date']=occupation['start_date_year']+'-'+occupation['start_date_month']+'-01' if all([occupation['start_date_year'],occupation['start_date_month']]) else ""
#             occupation['end_date']=occupation['end_date_year']+'-'+occupation['end_date_month']+'-01' if all([occupation['end_date_year'],occupation['end_date_month']]) else ""
#             occupation['country']=NameCodeConvert(country).getName(occupation['country']) # convert country code to country name
#             temp.append(occupation)
#         temp1=[]
#         for t in temp:
#             d={}
#             for k in ['start_date','end_date','job_title','company','city','province','country']:
#                 d[k]=t[k]
#             temp1.append(d) 
            
#         self.data['employment']=temp1    
    
#     # get same xx_type
#     def _getTableNodeWithType(self,table_node_list,the_type):
#         try:
#             for node in table_node_list:
#                 if node["variable_type"]==the_type:
#                     return node
#         except:
#             return None
    
#     # excel_file should be an existed excel file matching the 5257 program
#     def _makeExcel(self,excel_file,protection):
        
#         new_excel_obj=Excel(excel_file) #if path.isfile(excel_file) else self.new_excel_obj
#         # 根据表格的sheets/tables来处理。 
#         for sheet_name, sheet_obj in new_excel_obj.sheets.items():
#             for variable,variable_obj in sheet_obj.data.items():
#                 sheet=self.data.get(sheet_name)
#                 value=sheet.get(variable) if sheet else None
#                 new_excel_obj.sheets[sheet_name].data[variable].value=value
        
#         for table_name,table_obj in new_excel_obj.tables.items():
#             # this three tables will be processed independently
#             if (table_name in ['personid','address','phone']):
#                 for index,variable_obj in enumerate(table_obj.data):
#                     if variable_obj:
#                         node=self._getTableNodeWithType(self.data.get(table_name),variable_obj.variable_type)
#                         for k in variable_obj.__dict__.keys():
#                             value=node.get(k) if node else None
#                             if k not in ['variable_type','display_type']: setattr(new_excel_obj.tables[table_name].data[index],k,value)
#             # others are normal                
#             else:
#                 # 现在在每张表里，取出seld.data[table_name]的table_node数据字典，做成tablenode,放到表里的data中。
#                 node_data=self.data.get(table_name)
#                 if isinstance(node_data,dict):
#                     new_excel_obj.tables[table_name].data.append(TableNode(**node_data))
#                 elif isinstance(node_data,list):
#                     for node in node_data:
#                         new_excel_obj.tables[table_name].data.append(TableNode(**node))
#                 else:
#                     pass
#         # 根据最终更新了variables的sheets和talbes，生成新的excel文件
#         new_excel_obj.makeExcel(excel_file,protection=protection)

        
        
# #TODO: 明天需要做：
# # 1.  继续整理数据格式到excel。
# # 2. 根据5257模型中的sheet，生成excel文件，根据excel文件的sheets/tables，来这里找数据，然后给自己赋值。 然后保存






from .context import DATADIR
from typing import List
from model.common.commonmodel import CommonModel
from model.lmia.data import LmiaCase,Contact,General
from model.common.advertisement import Advertisement,InterviewRecord,InterviewRecords,Advertisements
from model.common.person import Person
from model.common.contact import Contacts
import os
from model.common.wordmaker import WordMaker

class Personal(Person):
    def __str__(self):
        return self.full_name
        
class RecruitmnetSummaryModel(CommonModel):
    general:General
    contact:List[Contact]
    lmiacase:LmiaCase
    advertisement:List[Advertisement]
    interviewrecord:List[InterviewRecord]
    
    # initialize the model with a list of excels, which includes all nececcery information the model required. if outpuot_excel_file is not None, it will make an excel file.
    def __init__(self,excels=None,output_excel_file=None):
        if output_excel_file:
            excels=self.getExcels(["excel/recruitment.xlsx","excel/lmia.xlsx","excel/er.xlsx"])
        else:
            if excels is None and len(excels)==0:
                raise ValueError('You must input excel file list as source data for validation')
        # call parent class for validating
        super().__init__(excels,output_excel_file,globals())
    
    @property
    def selected_contact(self):
        contacts=Contacts(self.contact)
        return contacts.preferredContact
    
    @property
    def summary(self):
        return InterviewRecords(self.interviewrecord)
    
    @property
    def advertisements(self):
        records=[]
        i=1
        for adv in self.advertisement:
            records.append({**{'days':adv.days},**adv.dict()})
            i+=1
        return records    
    
    
    

class RecruitmnetSummaryDocxAdaptor():
    def __init__(self,recruitment_summary_obj:RecruitmnetSummaryModel):
        self.recruitment_summary_obj=recruitment_summary_obj
        
    def re_generate_dict(self):
        summary_info={
            'resume_num':self.recruitment_summary_obj.summary.resume_num,
            'canadian_num':self.recruitment_summary_obj.summary.canadian_num,
            'unknown_num':self.recruitment_summary_obj.summary.unknown_num,
            "foreigner_num":self.recruitment_summary_obj.summary.foreigner_num,
            "total_canadian":self.recruitment_summary_obj.summary.total_canadian,
            "total_interviewed_canadians":self.recruitment_summary_obj.summary.total_interviewed_canadians,
            "canadian_records":self.recruitment_summary_obj.summary.canadian_records,
            'contact':self.recruitment_summary_obj.selected_contact,
            "advertisement":self.recruitment_summary_obj.advertisements
        }
        return {**self.recruitment_summary_obj.dict(),**summary_info}

    def make(self,output_docx,template_no=None):
        template_path=os.path.abspath(os.path.join(DATADIR,"word/lmia_recruitment_summary.docx"))      
        wm=WordMaker(template_path,self.re_generate_dict(),output_docx)
        wm.make()
    



import time
from abc import ABC, abstractmethod
from typing import Tuple
from datetime import date, timedelta
from utils.utils import best_match
from pywinauto.keyboard import send_keys


class Control(ABC):
    """Pdf form input control base class"""

    verbose: bool = False

    @abstractmethod
    def fill(self):
        """Fill this control"""


class Skip(Control):
    """Skip disabled control"""

    def fill(self):
        if self.verbose:
            print("Skip to next")
        send_keys("{TAB}")


class Button(Control):
    def fill(self):
        if self.verbose:
            print("Click button")
        send_keys("{ENTER}")


class Pause(Control):
    """Pause for a moment"""

    def __init__(self, seconds):
        self.seconds = seconds

    def fill(self):
        if self.verbose:
            print(f"Pause {self.seconds} seconds")
        time.sleep(self.seconds)


class TextField(Control):
    """Text field"""

    def __init__(self, data, pause=0):
        """data: text to fill"""
        self.data = data
        self.pause = pause

    def fill(self):
        if self.verbose:
            print(f"Fill text field with {self.data}")
        self.data = self.data or ""
        send_keys(self.data.replace(" ", "{SPACE}"), pause=self.pause)
        send_keys("{TAB}")


class RadioButton(Control):
    """Radio button"""

    def __init__(self, data):
        """data: False for No, True for yes"""
        self.data = data

    def fill(self):
        if self.data:
            key = "{VK_RIGHT}"
        else:
            key = "{VK_SPACE}"
        if self.verbose:
            print(f"Fill radio button with {self.data}, press {key} key")
        send_keys(key)
        send_keys("{TAB}")


class CheckBox(Control):
    """Check box"""

    def __init__(self, data):
        self.data = data

    def fill(self):
        if self.verbose:
            print(f"Fill check box with {self.data}")
        if self.data:
            send_keys("{VK_SPACE}")
            send_keys("{TAB}")
        else:
            send_keys("{TAB}")


class OutputInfo(Control):
    """Output log info"""

    def __init__(self, info):
        self.info = info

    def fill(self):
        if self.verbose:
            print(f"------- {self.info} -------")


class DropdownList(Control):
    """Dropdown list"""

    def __init__(self, data, options, like: bool = False):
        self.data = data
        self.options = options
        self.like = like

    def key_presses(self) -> Tuple[str, int]:
        s_char = self.data[0].lower()
        count = 0
        matched_one = best_match(self.data, self.options)
        for elem in self.options:
            if elem[0].lower() == s_char:
                count += 1
                if elem.lower() == matched_one.lower():
                    break
        return (s_char, count)

    def fill(self):
        key, num = self.key_presses()
        if self.verbose:
            print(f"Select dropdown list with {self.data}, press {key} {num} time(s)")
        for _ in range(num):
            send_keys(key)
        send_keys("{TAB}")


class DateField(Control):
    """Date field"""

    def __init__(self, the_date, noday: bool = False, pause=0.2):
        self.date = the_date or date.today() + timedelta(days=1)
        self.noday = noday
        self.pause = pause

    def fill(self):

        if self.noday:
            self.date = (
                self.date.strftime("%Y-%m") if type(self.date) == date else self.date
            )
            year, month = self.date.split("-")
            if self.verbose:
                print(f"Fill date {year}-{month}")
            send_keys(year, pause=self.pause)
            send_keys(month, pause=self.pause)
        else:
            self.date = (
                self.date.strftime("%Y-%m-%d") if type(self.date) == date else self.date
            )
            year, month, day = self.date.split("-")
            if self.verbose:
                print(f"Fill date {year}-{month}-{day}")
            send_keys(year, pause=self.pause)
            send_keys(month, pause=self.pause)
            send_keys(day, pause=self.pause)
        send_keys("{TAB}")



from webform.models.definition import Action
from webform.models.bcpnpmodel_reg import BcpnpModelReg, BcpnpEEModelReg
from webform.bcpnp.dashboard import DashboardReg
from typing import Union


class Registrant:
    def __init__(self, person: Union[BcpnpModelReg, BcpnpEEModelReg]):
        self.person = person

    def fill(self):
        current_previous_application = [
            {
                "action_type": Action.Radio.value,
                "label": "Do you currently have any other active registrations or applications with the BC Provincial Nominee Program?",
                "id": "#BCPNP_App_ActiveApplication-Yes"
                if self.person.bcpnp.q1
                else "#BCPNP_App_ActiveApplication-No",
            },
            {
                "action_type": Action.Radio.value,
                "label": "Have you applied to the BC Provincial Nominee Program in the past?",
                "id": "#BCPNP_App_PreviousApp-Yes"
                if self.person.bcpnp.has_applied_before
                else "#BCPNP_App_PreviousApp-No",
            },
        ]
        previous_file_number = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "Previous file number",
                    "id": "#BCPNP_App_CurPrevApplicationsDetails",
                    "value": self.person.bcpnp.pre_file_no,
                    "required": True,
                    "length": 100,
                }
            ]
            if self.person.bcpnp.has_applied_before
            else []
        )

        ee_profile = (
            [
                {
                    "action_type": Action.Input.value,
                    "label": "Express entry profile number",
                    "id": "#syncA_App_EE_ProfileNumber",
                    "value": self.person.ee.ee_profile_no,
                    "required": True,
                    "length": 30,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Express entry profile submission expiry date",
                    "id": "#syncA_App_EE_ExpiryDate",
                    "value": self.person.ee.ee_expiry_date.strftime("%Y-%m-%d"),
                    "required": True,
                    "length": 10,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Job seeker validation code (JSVC)",
                    "id": "#syncA_App_EE_ValidCode",
                    "value": self.person.ee.ee_jsvc,
                    "required": True,
                    "length": 30,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Comprehensive ranking score (CRS)",
                    "id": "#syncA_App_EE_CRS",
                    "value": self.person.ee.ee_score,
                    "required": True,
                    "length": 30,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "NOC (as supplied on your IRCC Express Entry profile)",
                    "id": "#syncA_App_Job_NOC_EE",
                    "value": self.person.ee.ee_noc,
                    "required": True,
                    "length": 30,
                },
                {
                    "action_type": Action.Input.value,
                    "label": "Job title (as supplied on your IRCC Express Entry profile)",
                    "id": "#syncA_App_Job_Title_EE",
                    "value": self.person.ee.ee_job_title,
                    "required": True,
                    "length": 30,
                },
            ]
            if type(self.person) == BcpnpEEModelReg
            else []
        )

        # dashboard = DashboardReg()

        actions = (
            # dashboard.jump("Registrant")
            current_previous_application
            + previous_file_number
            + ee_profile
            # + dashboard.save
        )
        return [
            {
                "action_type": Action.WebPage.value,
                "page_name": "Registrant",
                "actions": actions,
                "id": "body > div > main > div.layout-container > div > div > div > uf-form > div > ng-transclude > div > uf-tabset > div > ul > li:nth-child(2) > a",
                "next_page_tag":"#BCPNP_App_Edu_CA-No"
            }
        ]



from pydantic import BaseModel
from engine.scoring import getKeyValuePoint,getRangePoint,getNOCLevel
from policy.bcpnp import data
from dataclasses import dataclass

class ScoringBCPNP(BaseModel):
    noc:str='1111'
    is_working_in_the_position:bool=False
    has_one_year_canadian_experience:bool=False
    hourly_rate:float=25
    working_hours:float=40
    work_experience:int=24
    region:str='Greater Vancouver'
    education:str='Bachelor'
    education_bonus:str='ECA'
    clb:int=4

    @property
    def skill_level(self):
        return getNOCLevel(self.noc)
    
    @property
    def noc_level_point(self):
        return getKeyValuePoint(data.skill_level_points,self.skill_level)
    
    @property
    def noc_00_point(self):
        return getKeyValuePoint(data.skill_level_bonus_points,self.skill_level)
    
    @property
    def wage_point(self):
        return getRangePoint(data.wage_points,int(self.hourly_rate*self.working_hours*52))
    
    @property
    def work_experience_point(self):
        return getRangePoint(data.work_experience_points,self.work_experience)
    
    @property
    def working_in_the_position_point(self):
        return getKeyValuePoint(data.working_in_the_position_points,self.is_working_in_the_position)
    
    @property
    def one_year_Canadian_work_experience_point(self):
        return getKeyValuePoint(data.one_year_experience_point,self.has_one_year_canadian_experience)
    
    @property
    def educaiton_level_point(self):
        return getKeyValuePoint(data.education_points,self.education)
    
    @property
    def education_bonus_point(self):
        return getKeyValuePoint(data.education_bonus_points,self.education_bonus)
    
    @property
    def region_point(self):
        return getKeyValuePoint(data.region_points,self.region)
    
    @property
    def language_level_point(self):
        return getKeyValuePoint(data.language_points,self.clb)
    
    @property
    def total_point(self):
        return (self.noc_level_point+self.noc_00_point+self.wage_point+self.work_experience_point+
                self.working_in_the_position_point+self.one_year_Canadian_work_experience_point+
                self.educaiton_level_point+self.education_bonus_point+self.region_point+self.language_level_point)